import R from 'ramda';
import React, {PureComponent} from 'react';
import {connect} from 'react-redux';
import classNames from 'classnames';
import PropTypes from 'prop-types';
import buttonActions from '../actions/button-actions';
import {PrimaryButton} from './items/primary-button';
import {DebouncedButton} from './items/debounced-button';
import {SecondaryButton} from './items/secondary-button';
import {PrimaryDropdownOnlyButton} from './items/primary-dropdown-only-button';
import {SecondaryDropdownOnlyButton} from './items/secondary-dropdown-only-button';
import {ToggleButton} from './items/toggle-button';

import './buttons-page.less';

const mapStateToProps = (state) => ({buttonsContext: state.button});

const mapDispatchToProps = {updateModel: buttonActions.updateModel};

@connect(mapStateToProps, mapDispatchToProps)
export class Buttons extends PureComponent {
    static propTypes = {
        buttonsContext: PropTypes.object,
        match: PropTypes.object,
        updateModel: PropTypes.func
    };

    onUpdateModel = (modelName, value) => () => this.props.updateModel(modelName, value);

    render() {
        const buttons = {
            debouncedButton1: <DebouncedButton
                key={'debouncedButton1'}
                modelName={'debouncedButton1'}
                state={this.props.buttonsContext}
                updateModel={this.onUpdateModel}>
                <span>Increase counter</span>
            </DebouncedButton>,
            primaryButton: <PrimaryButton
                key={'primaryButton'}
                modelName={'primaryButton'}
                state={this.props.buttonsContext}
                updateModel={this.onUpdateModel}
            />,
            primaryDropdownOnlyButton: <PrimaryDropdownOnlyButton
                key={'primaryDropdownOnlyButton'}
                modelName={'primaryDropdownOnlyButton'}
                state={this.props.buttonsContext}
                updateModel={this.onUpdateModel}
            />,
            secondaryButton: <SecondaryButton
                key={'secondaryButton'}
                modelName={'secondaryButton'}
                state={this.props.buttonsContext}
                updateModel={this.onUpdateModel}
            />,
            secondaryDropdownOnlyButton: <SecondaryDropdownOnlyButton
                key={'secondaryDropdownOnlyButton'}
                modelName={'secondaryDropdownOnlyButton'}
                state={this.props.buttonsContext}
                updateModel={this.onUpdateModel}
            />,
            toggleButton: <ToggleButton
                key={'toggleButton'}
                modelName={'toggleButton'}
                state={this.props.buttonsContext}
                updateModel={this.onUpdateModel}
            />
        };

        const {match} = this.props;
        const isSingleItem = match && buttons[match.params.name];

        return <div className={classNames('buttons-page', {'single-item': isSingleItem})}>
            <div className="page-content">
                {isSingleItem ? buttons[match.params.name] : R.values(buttons)}
            </div>
        </div>;
    }
}
