import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import AlertSwitcher from '../widget-page/alert-switcher';
import {XlReactWidgetAlertMessage} from '../../xl-react-widget/xl-react-widget-alert-message/xl-react-widget-alert-message'; // eslint-disable-line max-len
import ElementNavTitle from './element-nav-title';

export default class AlertMessageTestComponent extends PureComponent {
    static propTypes = {
        className: PropTypes.string.isRequired,
        currentState: PropTypes.object.isRequired,
        modelName: PropTypes.string.isRequired,
        updateState: PropTypes.func
    };

    alertSwitcher = (updateState, modelName) => {
        const {currentState} = this.props;
        const handleChangeType = (newType) => updateState(modelName, {...currentState[modelName], alertType: newType});

        return <AlertSwitcher
            alertTypes="success,warning,error"
            changeType={handleChangeType}
            chosenType={currentState[modelName].alertType}
            groupName={modelName}
        />;
    };

    render() {
        const {className, currentState, modelName, updateState} = this.props;
        const {alertType, navigation: {description, title, type}} = currentState[modelName];

        return <div>
            <ElementNavTitle
                description={description}
                modelName={modelName}
                title={title}
                type={type}
            />
            <div className={className}>
                <div className="testComponent">
                    <XlReactWidgetAlertMessage internalState={alertType}>
                        Something went terribly wrong
                    </XlReactWidgetAlertMessage>
                    {this.alertSwitcher(updateState, modelName)}
                </div>
            </div>
        </div>;
    }
}
