/* eslint-disable max-lines */
import R from 'ramda';
import localeMessages from '../../../languages/en-US/react-components.lang';
import {widgetStringValues} from '../../../xl-react-widget/xl-react-widget-string/xl-react-widget-string-enum';
import kitchenSinkActions from '../../actions/kitchen-sink-actions';
import {bindingTypes} from './component-bindings';

export const displayModelBinding = {
    className: 'model',
    label: localeMessages.modelLabel,
    name: 'model',
    type: bindingTypes.DISPLAY
};

export const displayPristineBinding = (name = 'pristine') => ({
    className: 'pristine',
    label: localeMessages.pristineLabel,
    name,
    type: bindingTypes.DISPLAY
});

export const displayValidBinding = (name = 'valid') => ({
    className: 'valid',
    label: localeMessages.validLabel,
    name,
    type: bindingTypes.DISPLAY
});

export const displayValueBinding = {
    className: 'value',
    label: localeMessages.valueLabel,
    name: 'value',
    type: bindingTypes.DISPLAY
};

export const displaySelectedLinesBinding = {
    className: 'selected-lines',
    label: localeMessages.selectedLinesLabel,
    name: 'selectedLines',
    type: bindingTypes.DISPLAY
};

export const displaySelectedActionBinding = {
    className: 'selected-action',
    label: localeMessages.selectedActionLabel,
    name: 'selectedAction',
    type: bindingTypes.DISPLAY
};

export const displayFileTypeBinding = {
    className: 'file-type',
    label: localeMessages.fileTypeLabel,
    name: 'fileType',
    type: bindingTypes.DISPLAY
};

export const displayLastPushedPatchBinding = {
    className: 'last-pushed-patch',
    label: localeMessages.lastPushedPatchLabel,
    name: 'lastPushedPatch',
    type: bindingTypes.DISPLAY
};

export const apiBinding = {
    className: 'api',
    label: localeMessages.apiLabel,
    name: 'api',
    type: bindingTypes.API
};

export const booleanDisabledBinding = {
    className: 'xl-disabled',
    label: localeMessages.disabledLabel,
    name: 'disabled',
    type: bindingTypes.BOOLEAN,
    value: false
};

export const booleanRequiredBinding = (value) => ({
    className: 'required',
    label: localeMessages.requiredLabel,
    name: 'metadata.required',
    type: bindingTypes.BOOLEAN,
    value
});

export const booleanShowMessagesBinding = {
    action: {type: kitchenSinkActions.TOGGLE_MESSAGE_VISIBILITY},
    className: 'show-messages',
    label: localeMessages.showMessagesLabel,
    name: 'showMessages',
    type: bindingTypes.BOOLEAN,
    value: false
};

export const booleanStrippedBinding = {
    className: 'stripped',
    label: localeMessages.strippedLabel,
    name: 'stripped',
    type: bindingTypes.BOOLEAN,
    value: false
};

export const booleanAllowResizeBinding = {
    className: 'xl-allow-resize',
    label: localeMessages.allowResize,
    name: 'allowResize',
    type: bindingTypes.BOOLEAN,
    value: true
};

export const booleanAnimateBinding = {
    className: 'animation',
    label: localeMessages.animateLabel,
    name: 'animation',
    type: bindingTypes.BOOLEAN,
    value: false
};

export const booleanShowModalBinding = (propName) => ({
    className: 'show-modal',
    label: localeMessages.showButton,
    name: propName,
    type: bindingTypes.BOOLEAN,
    value: false
});

export const booleanOnlyUniqueSuggestionBinding = {
    className: 'only-unique-suggestion',
    label: localeMessages.onlyUniqueSuggestionLabel,
    name: 'onlyUniqueSuggestion',
    type: bindingTypes.BOOLEAN,
    value: false
};

export const booleanReadOnlyBinding = {
    className: 'readonly',
    label: localeMessages.readOnlyLabel,
    name: 'options.readOnly',
    type: bindingTypes.BOOLEAN,
    value: true
};

export const enumDisplayModeBinding = (value = '') => ({
    className: 'display-mode',
    enum: [{
        className: 'creation',
        label: 'Creation',
        value: 'creation'
    }, {
        className: 'edit',
        label: 'Edit',
        value: 'edit'
    }, {
        className: 'display',
        label: 'Display',
        value: 'visual'
    }],
    label: localeMessages.displayModeLabel,
    name: 'displayMode',
    type: bindingTypes.ENUM,
    value
});

export const enumInputTypeBinding = {
    className: 'input-type',
    enum: [{
        className: 'password',
        label: 'password',
        value: 'password'
    }, {
        className: 'text',
        label: 'text',
        value: 'text'
    }],
    label: localeMessages.inputTypeLabel,
    name: 'metadata.type',
    type: bindingTypes.ENUM,
    value: 'text'
};

export const enumFileTypeBinding = {
    className: 'file-type',
    enum: [{
        className: 'yaml',
        label: 'yaml',
        value: 'yaml'
    }, {
        className: 'json',
        label: 'json',
        value: 'json'
    }],
    label: localeMessages.fileTypeLabel,
    name: 'metadata.fileType',
    type: bindingTypes.ENUM,
    value: 'yaml'
};

export const enumProgressTypeBinding = {
    className: 'progress-type',
    enum: [{
        className: 'progress',
        label: 'progress',
        value: 'progress'
    }, {
        className: 'success',
        label: 'success',
        value: 'success'
    }, {
        className: 'error',
        label: 'error',
        value: 'error'
    }],
    label: localeMessages.typeLabel,
    name: 'currentStatus',
    type: bindingTypes.ENUM,
    value: 'progress'
};

export const enumSkinBinding = {
    className: 'skin',
    enum: [{
        className: 'checkbox-skin',
        label: 'checkbox',
        value: 'checkbox'
    }, {
        className: 'color-and-checkbox-skin',
        label: 'color-and-checkbox',
        value: 'color-and-checkbox'
    }],
    label: localeMessages.skinLabel,
    name: 'skin',
    type: bindingTypes.ENUM,
    value: 'checkbox'
};

export const enumConfirmationDialogSkinBinding = {
    className: 'skin',
    enum: [{
        className: 'checkbox-skin',
        label: 'checkbox',
        value: 'checkbox'
    }, {
        className: 'default',
        label: 'default',
        value: 'default'
    }],
    label: localeMessages.skinLabel,
    name: 'skin',
    type: bindingTypes.ENUM,
    value: 'default'
};

export const inputChangeViewBinding = (values) => ({
    className: 'change-view',
    label: 'View',
    model: '',
    name: 'metadata.view',
    type: bindingTypes.SELECT,
    values: R.values(values)
});

export const inputSizeBinding = {
    className: 'change-size',
    label: 'Size',
    model: '',
    name: 'metadata.size',
    type: bindingTypes.SELECT,
    values: R.values(widgetStringValues)
};

export const inputDefaultValueBinding = {
    buttonLabel: localeMessages.applyDefaultValueButton,
    className: 'default-value',
    label: 'Default value',
    name: 'metadata.default',
    onChangeCallApi: 'applyDefaultModel',
    type: bindingTypes.INPUT,
    value: ''
};

export const integerCurrentProgressBinding = {
    className: 'current-progress',
    label: localeMessages.currentProgressLabel,
    name: 'currentProgress',
    type: bindingTypes.INT
};

export const minimumDateBinding = (value) => ({
    className: 'date',
    label: localeMessages.minimumDateLabel,
    name: 'minDate',
    type: bindingTypes.DATE,
    value
});

export const maximumDateBinding = (value) => ({
    className: 'date',
    label: localeMessages.maximumDateLabel,
    name: 'maxDate',
    type: bindingTypes.DATE,
    value
});

export const actionBinding = (name, actionDescriptor) => ({
    className: 'show-modal',
    label: name,
    name: 'reduxAction',
    type: bindingTypes.ACTION,
    value: actionDescriptor
});

export const commonBindings = (displayMode = '', required = false, pristineName = 'pristine', validName = 'valid') => [
    displayModelBinding,
    displayPristineBinding(pristineName),
    displayValidBinding(validName),
    booleanDisabledBinding,
    booleanRequiredBinding(required),
    booleanShowMessagesBinding,
    enumDisplayModeBinding(displayMode),
    apiBinding
];
