import React, {Component} from 'react';
import PropTypes from 'prop-types';
import {isComponentDisabled} from '../../utils/components-utils';
import RU from '../../utils/ramda-utils';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {XlReactDipCiComponent} from './xl-react-dip-ci-component';

export class XlReactDipCi extends Component {
    static propTypes = {
        className: PropTypes.string,
        controlClassName: PropTypes.string,
        handlers: PropTypes.object,
        model: PropTypes.string,
        objLabel: PropTypes.string,
        showSearchIcon: PropTypes.bool,
        toggleButton: PropTypes.node,
        ...XlReactDipContainerProps
    };

    static defaultProps = {displayMode: 'edit', metadata: {}, objLabel: 'id', validators: []};

    constructor(props) {
        super(props);
        props.onModelChange(props.model, false);
    }

    componentWillUnmount() {
        RU.invoke('unregister', [], this.props);
    }

    onWidgetModelChange = (model) => {
        this.props.onModelChange(model);
    };

    messageHandler = (message) => {
        this.setState({message});
    };

    render() {
        const {
            className,
            controlClassName,
            handlers,
            message,
            metadata,
            model,
            objLabel,
            onMessageChange,
            onPristinityChange,
            onValidityChange,
            pristine,
            register,
            toggleButton,
            showSearchIcon,
            valid
        } = this.props;
        const disabled = isComponentDisabled(this.props);

        return <XlReactDipCiComponent {...({
            className,
            controlClassName,
            disabled,
            handlers,
            message,
            messageHandler: this.messageHandler,
            metadata,
            model,
            objLabel,
            onMessageChange,
            onModelChange: this.onWidgetModelChange,
            onPristinityChange,
            onValidityChange,
            pristine,
            register,
            showSearchIcon,
            toggleButton,
            valid
        })}
        />;
    }
}
