import R from 'ramda';
import RU from '../../utils/ramda-utils';
import {transformItem, transformModel} from '../../utils/transform-utils';
import {WARNING, withoutRootNode} from '../../utils/ci-utils';
import {localeMessages} from '../../services/locale-service';

export const invalidCiValidator = (context) => () => {
    const invalidCi = R.find(R.propEq('valid', false), context.validatedItems);
    return invalidCi ? {
        condition: true,
        message: {
            show: false,
            text: localeMessages.invalidCiInTheListMessage,
            type: WARNING
        }
    } : {};
};

export const sortItems = (items, objLabel = 'id') => {
    if (!items || R.isEmpty(items)) {
        return items;
    }
    const itemsToSort = R.map((item) => ({
        origin: item,
        sortKey: withoutRootNode(R.is(String, item) ? item : item[objLabel])
    }), items);
    const sortedItems = R.sortBy(R.prop('sortKey'))(itemsToSort);
    return R.pluck('origin', sortedItems);
};

export const prepareViewModel = (items, objLabel, sort) => R.pipe(
    (list) => sort ? sortItems(list, objLabel) : list,
    transformModel(objLabel),
    RU.indexedArray,
    R.map((model) => ({model, valid: true}))
)(items);

export const filterItems = (items, searchTerm = '', objLabel) => searchTerm ? R.filter(
    R.pipe(
        R.prop('model'),
        R.head,
        transformItem(R.__, objLabel),
        R.toLower,
        R.includes(R.toLower(searchTerm))
    ), items) : items;

export const addItem = (items, item, unique) => {
    if (!unique || !R.includes(item, items)) {
        return [...items, item];
    }
    return items;
};

export const moveItem = (from, to, items) => R.pipe(
    R.remove(from, 1),
    R.insert(to, items[from])
)(items);
