import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import classNames from 'classnames';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {XlReactWidgetAutocomplete} from '../../xl-react-widget/xl-react-widget-autocomplete/xl-react-widget-autocomplete'; // eslint-disable-line max-len
import {XlReactFormItemControl} from '../../xl-react-form/xl-react-form-item-control';
import {isValidEnumValue} from '../../services/validate-service';
import {validationMessageClasses} from '../../utils/css-utils';
import {getInputHintValues, hasInputHints} from '../../utils/ci-utils';

import './xl-react-dip-enum.less';
import {XlReactWidgetRadioGroup} from '../../xl-react-widget/xl-react-widget-radio-group/xl-react-widget-radio-group';
import {createRadioGroupModel, enumViews, getHandlers, handleModelChange} from './dip-enum-service';

export class XlReactDipEnumComponent extends PureComponent {
    static propTypes = {
        disabled: PropTypes.bool,
        message: xlMessageTypeProp,
        metadata: PropTypes.object,
        model: PropTypes.string,
        onModelChange: PropTypes.func,
        registerChildApi: PropTypes.func,
        valid: PropTypes.bool,
        view: PropTypes.string
    };

    static defaultProps = {view: enumViews.DEFAULT};

    getEnumValues = () => this.isInputHintsPresent ?
        getInputHintValues(this.props.metadata) : this.props.metadata.enumValues;

    render() {
        const {
            disabled,
            message,
            metadata,
            model,
            onModelChange,
            registerChildApi,
            valid,
            view
        } = this.props;

        this.isInputHintsPresent = hasInputHints(metadata);
        const enumValues = this.getEnumValues();
        const invalid = !isValidEnumValue({checkedValue: model, enumValues});
        const handlers = getHandlers(this.isInputHintsPresent, enumValues);
        const controlClass = classNames({'xl-react-dip-enum': true}, validationMessageClasses(valid, message));
        const handleChangeModel = handleModelChange(view, this.isInputHintsPresent, enumValues, onModelChange);

        const pickComponentToRender = () => R.cond([
            [R.equals(enumViews.RADIO), () => <XlReactWidgetRadioGroup
                disabled={disabled}
                independent={true}
                model={createRadioGroupModel(enumValues, model)}
                name={metadata.name}
                onModelChange={handleChangeModel}
                register={registerChildApi}
                {...metadata}
            />],
            [R.T, () => <XlReactWidgetAutocomplete {...({
                disabled,
                handlers,
                independent: true,
                invalid,
                message: R.omit(['text'], message),
                metadata: R.omit(['label', 'description'], metadata),
                model,
                objLabel: this.isInputHintsPresent ? 'label' : 'id',
                onModelChange: handleChangeModel,
                register: registerChildApi,
                valid
            })}
            />]
        ])(view);

        return <XlReactFormItemControl {...({...metadata, controlClass, message})}>
            {pickComponentToRender()}
        </XlReactFormItemControl>;
    }
}
