import R from 'ramda';
import React from 'react';
import PropTypes from 'prop-types';
import {Column} from 'react-virtualized';
import {localeMessages} from '../../../services/locale-service';
import {InputKey} from '../input-key';
import {COLUMNS, shouldFocusInput} from '../map-table-utils';
import {InlineKeyInput} from '../../input/inline-key-input';
import {headerRenderer} from './header-renderer';

export const tableKeyColumn = (props) => {
    const {
        assignRef,
        cellState,
        columnWidth,
        disabled,
        handleKeyBlur,
        handleNewKeyBlur,
        handleNewKeyChange,
        handleNewKeyFocus,
        invalid,
        newKey,
        showNewKeyValueRow,
        updateCellState
    } = props;

    const cellRenderer = (cellProps) => {
        const {rowIndex, cellData} = cellProps;

        const focusInput = shouldFocusInput(rowIndex, COLUMNS.key, cellState);

        const handleUpdateCellState = (value) => {
            updateCellState({column: COLUMNS.key, row: rowIndex, value});
        };

        const onFocusNewKey = (value, event) => {
            handleUpdateCellState(value);
            handleNewKeyFocus(event);
        };

        const shouldDisplayInputKey = () => R.and(R.equals(rowIndex, 0), showNewKeyValueRow);

        if (shouldDisplayInputKey()) {
            return <InputKey
                assignRef={assignRef}
                disabled={disabled}
                handleNewKeyBlur={handleNewKeyBlur}
                handleNewKeyChange={handleNewKeyChange}
                handleNewKeyFocus={onFocusNewKey}
                invalid={invalid}
                model={newKey}
                shouldFocus={focusInput}
            />;
        }

        const getChangedValue = () => focusInput ? cellState.value : undefined;

        return !R.isNil(cellData) && <InlineKeyInput
            changedValue={getChangedValue()}
            onBlur={handleKeyBlur}
            readOnly={disabled}
            shouldFocus={focusInput}
            updateCellState={handleUpdateCellState}
            value={cellData}
        />;
    };

    return <Column
        cellRenderer={cellRenderer}
        className="xl-virtualized-table-cell"
        dataKey="key"
        flexGrow={1}
        headerRenderer={headerRenderer(localeMessages.keyLabel, COLUMNS.key)}
        width={columnWidth}
    />;
};

tableKeyColumn.propTypes = {
    assignRef: PropTypes.func.isRequired,
    cellState: PropTypes.shape({
        column: PropTypes.oneOf([COLUMNS.key, COLUMNS.value]).isRequired,
        row: PropTypes.number.isRequired,
        value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired
    }),
    columnWidth: PropTypes.number.isRequired,
    disabled: PropTypes.bool.isRequired,
    handleKeyBlur: PropTypes.func.isRequired,
    handleNewKeyBlur: PropTypes.func.isRequired,
    handleNewKeyChange: PropTypes.func.isRequired,
    handleNewKeyFocus: PropTypes.func.isRequired,
    invalid: PropTypes.bool.isRequired,
    newKey: PropTypes.string,
    showNewKeyValueRow: PropTypes.bool.isRequired,
    updateCellState: PropTypes.func.isRequired,
    valid: PropTypes.bool.isRequired
};
