import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import {localeMessages} from '../../services/locale-service';
import {blurFieldOnSubmit} from '../input/input-service';
import XlReactWidgetRevealPasswordIcon
    from '../../xl-react-widget/xl-react-widget-reveal-password-icon/xl-react-widget-reveal-password-icon';
import {getInputType}
    from '../../xl-react-widget/xl-react-widget-reveal-password-icon/xl-react-widget-reveal-password-utils';
import {NEW_VALUE_CELL_KEY} from './map-table-utils';

export class InputValue extends PureComponent {
    static propTypes = {
        disabled: PropTypes.bool.isRequired,
        handleCloseButton: PropTypes.func.isRequired,
        handleFocus: PropTypes.func.isRequired,
        handleNewValueBlur: PropTypes.func.isRequired,
        handleNewValueKeyDown: PropTypes.func.isRequired,
        handleRevealPassword: PropTypes.func.isRequired,
        inputType: PropTypes.oneOf(['password', 'text']).isRequired,
        model: PropTypes.string,
        revealPassword: PropTypes.bool.isRequired,
        shouldFocus: PropTypes.bool.isRequired,
        showRemoveButton: PropTypes.bool.isRequired
    };

    componentDidMount() {
        if (this.props.shouldFocus && this.inputField) {
            this.inputField.focus({preventScroll: true});
        }
    }

    handleKeyDown = (event) => {
        blurFieldOnSubmit(this.inputField, event);
    };

    assignInputValueRef = (input) => {
        this.inputField = input;
    };

    shouldShowIcon = () => {
        const {model, inputType} = this.props;
        return model && inputType === 'password';
    };

    inputValueInput = (
        {
            disabled,
            handleCloseButton,
            handleNewValueBlur,
            handleFocus,
            handleNewValueKeyDown,
            handleRevealPassword,
            inputType,
            revealPassword,
            model,
            showRemoveButton
        }
    ) =>
        <div className="value-div new-value-div">
            {inputType === 'password' ?
                <input
                    disabled={disabled}
                    name="add-value"
                    onBlur={handleNewValueBlur}
                    onChange={handleNewValueKeyDown}
                    onFocus={handleFocus(NEW_VALUE_CELL_KEY)}
                    onKeyDown={this.handleKeyDown}
                    placeholder={localeMessages.valuePlaceholder}
                    ref={this.assignInputValueRef}
                    type={getInputType(inputType, revealPassword)}
                    value={model}
                /> :
                <textarea
                    disabled={disabled}
                    name="add-value"
                    onBlur={handleNewValueBlur}
                    onChange={handleNewValueKeyDown}
                    onFocus={handleFocus(NEW_VALUE_CELL_KEY)}
                    onKeyDown={this.handleKeyDown}
                    placeholder={localeMessages.valuePlaceholder}
                    ref={this.assignInputValueRef}
                    value={model}
                />
            }
            {
                showRemoveButton &&
                <i
                    className="xl-icon close-icon"
                    onClick={handleCloseButton}
                />
            }
            <XlReactWidgetRevealPasswordIcon
                handleRevealPassword={handleRevealPassword(NEW_VALUE_CELL_KEY)}
                isPasswordRevealed={revealPassword}
                showIconCondition={this.shouldShowIcon}
            />
        </div>;

    render() {
        const inputValueSpan = <span className="input-value">{localeMessages.valuePlaceholder}</span>;
        return this.props.disabled ? inputValueSpan : this.inputValueInput(this.props);
    }
}
