import R from 'ramda';

export const externalModelToInternal = (model = {}) => {
    const externalProperties = model['external-properties'];
    const keys = R.keys(externalProperties || {});
    const props = R.reduce((acc, key) => ({...acc, [key]: externalProperties[key]}), {}, keys);
    return {...R.omit(['external-properties'], model), ...props};
};

export const internalModelToExternal = (model, externalProperties) => {
    const section = externalProperties ?
        {'external-properties': R.reduce((acc, item) => ({...acc, [item]: model[item]}), {}, externalProperties)} : {};
    return {...R.omit([externalProperties], model), ...section};
};

export const getExternalPropertyKeys = R.pipe(
    R.propOr({}, 'external-properties'),
    R.keys
);

const getModel = (isOn, memorizedModel) => {
    if (memorizedModel) {
        return memorizedModel;
    }

    return isOn ? {key: '', kind: 'lookup', provider: ''} : undefined;
};

export const transformModelOnToggle = (model, ciPropName, isOn, memorizedModel) => {
    const fallbackModel = getModel(isOn, memorizedModel);

    if (isOn) {
        return R.pipe(
            R.assoc(ciPropName, undefined),
            R.assocPath(['external-properties', ciPropName], fallbackModel)
        )(model);
    }
    const newModel = R.pipe(
        R.dissocPath(['external-properties', ciPropName]),
        R.assoc(ciPropName, memorizedModel)
    )(model);

    if (R.isEmpty(R.prop('external-properties', newModel))) {
        return R.pipe(
            R.assoc(ciPropName, memorizedModel),
            R.assoc('external-properties', undefined)
        )(newModel);
    }

    return newModel;
};
