import React, {Component} from 'react';
import PropTypes from 'prop-types';
import {connect} from 'react-redux';
import {XlReactDipDynamicForm} from '../xl-react-dip-dynamic-form';
import formActions from './xl-react-dip-redux-form-actions';
import {getFormById} from './xl-react-dip-form-selectors';

const mapStateToProps = (state, ownProps) => getFormById(state, ownProps.formId);

const mapDispatchToProps = (dispatch, ownProps) => ({
    mergeContext: (context) => dispatch(formActions.mergeContext(ownProps.formId, context)),
    openCategoryGroup: (categoryName) => dispatch(formActions.openCategoryGroup(ownProps.formId, categoryName)),
    removeValidationMessages:
        (propertyName) => dispatch(formActions.removeValidationMessages(ownProps.formId, propertyName)),
    toggleCategoryGroup: (categoryName) => dispatch(formActions.toggleCategoryGroup(ownProps.formId, categoryName)),
    unmount: () => dispatch(formActions.unmount(ownProps.formId)),
    updateExternalProperties: (externalProperties) =>
        dispatch(formActions.updateExternalProperties(ownProps.formId, externalProperties))
});

@connect(mapStateToProps, mapDispatchToProps)
export class XlReactDipReduxForm extends Component {
    static propTypes = {
        autoCleanup: PropTypes.bool,
        unmount: PropTypes.func
    };

    static defaultProps = {autoCleanup: true};

    componentWillUnmount() {
        const {autoCleanup, unmount} = this.props;
        if (autoCleanup) {
            unmount();
        }
    }

    render() {
        return <XlReactDipDynamicForm {...this.props}/>;
    }
}
