import $ from 'jquery';
import R from 'ramda';
import ResizeObserver from 'resize-observer-polyfill';

export const findTabSystem = (tabSystemId) => $(`.xl-react-tab-system.${tabSystemId}`);
export const findDropDown = (tabSystemId) =>
    findTabSystem(tabSystemId).find('.header-column .btn-group-backdrop > .dropdown');
export const getDropDownWidth = (tabSystemId) => findDropDown(tabSystemId).width() + 10 || 0;

const getAddNewTabButton = (tabSystemId) =>
    findTabSystem(tabSystemId).find('.header-column .btn-group-backdrop > .btn-add-new-tab');
const addNewTabButtonWidth = (tabSystemId) => getAddNewTabButton(tabSystemId).width() + 10 || 0;

const getButtonGroup = (tabSystemId) => findTabSystem(tabSystemId).find('.header-column > .btn-group-backdrop');

const activeTab = (tabSystemId) => findTabSystem(tabSystemId).find('li[role="presentation"].active');
const getActiveTabWidth = (tabSystemId) => activeTab(tabSystemId).width();
const getActiveTabLeftPosition = (tabSystemId) => activeTab(tabSystemId).position() ?
    activeTab(tabSystemId).position().left : 0;

export const repositionElementsInTabNavigation = (tabSystemId) => {
    const tabSystemPanel = findTabSystem(tabSystemId);
    const navigationTabPanel = tabSystemPanel.find('.nav.nav-tabs');
    const activeTabWidth = getActiveTabWidth(tabSystemId);
    const positionOfActiveTab = getActiveTabLeftPosition(tabSystemId);
    const dropDownWidth = getDropDownWidth(tabSystemId);
    const headerColumn = tabSystemPanel.find('.header-column');
    const headerColumnWidth = headerColumn.width();
    const addNewTabBtnWidth = addNewTabButtonWidth(tabSystemId);
    const buttonGroup = getButtonGroup(tabSystemId);

    const offsetFromPosition = positionOfActiveTab + activeTabWidth + dropDownWidth + addNewTabBtnWidth;
    const navigationTabPanelOffset = headerColumnWidth - offsetFromPosition - 2;
    const isNotEnoughWidth = offsetFromPosition > headerColumnWidth;
    navigationTabPanel.css('left', isNotEnoughWidth ? navigationTabPanelOffset : 0);

    const navigationTabPanelWidth = navigationTabPanel.width();
    const isAddButtonOverflowing = navigationTabPanelWidth + addNewTabBtnWidth + dropDownWidth >= headerColumnWidth;

    if (isAddButtonOverflowing) {
        buttonGroup.css({width: addNewTabBtnWidth + dropDownWidth});
        buttonGroup.addClass('with-shadow');
    } else {
        buttonGroup.css('width', headerColumnWidth - navigationTabPanelWidth);
        buttonGroup.removeClass('with-shadow');
    }
};

const resizeObserverInstance = new ResizeObserver((entries) => {
    entries.forEach((entry) => {
        const tabSystemId = $(entry.target).data('tabsystemid');
        repositionElementsInTabNavigation(tabSystemId);
    });
});

export const handleTabResize = (tabSystemId) => {
    const tabSystemPanel = findTabSystem(tabSystemId);
    resizeObserverInstance.observe(R.head(tabSystemPanel));
};

export const handleTabUnmount = () => resizeObserverInstance.disconnect();
