import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import classNames from 'classnames';
import RU from '../../../utils/ramda-utils';

export class XlReactWidgetAutocompleteInput extends PureComponent {
    static propTypes = {
        assignInputRef: PropTypes.func,
        disabled: PropTypes.bool,
        handleErase: PropTypes.func,
        inputProps: PropTypes.object,
        onInputBlur: PropTypes.func,
        onInputChange: PropTypes.func,
        onInputFocus: PropTypes.func,
        onKeyDown: PropTypes.func,
        onShowAllOptions: PropTypes.func,
        placeholder: PropTypes.string,
        showDropDownTriangle: PropTypes.bool,
        showSearchIcon: PropTypes.bool,
        value: PropTypes.string
    };

    static defaultProps = {disabled: false, inputProps: {}, placeholder: '', value: ''};

    handleOnBlur = (event) => {
        RU.invoke('onInputBlur', [event], this.props);
    };

    handleOnChange = (event) => {
        RU.invoke('onInputChange', [event], this.props);
    };

    handleOnFocus = (event) => {
        RU.invoke('onInputFocus', [event], this.props);
    };

    handleKeyDown = (event) => {
        RU.invoke('onKeyDown', [event], this.props);
    };

    render() {
        const {
            disabled,
            handleErase,
            inputProps,
            onShowAllOptions,
            placeholder,
            assignInputRef,
            showSearchIcon,
            showDropDownTriangle,
            value
        } = this.props;

        const closeField = () => !R.isEmpty(value) && !disabled &&
            <i className="xl-icon close-icon" onClick={handleErase}/>;

        const showAll = () => <i className="xl-icon arrow-down-icon show-all" onClick={onShowAllOptions}/>;

        return (
            <span className={classNames({'with-search': showSearchIcon})}>
                {showSearchIcon && <i className="xl-icon search-icon"/>}
                <input
                    {...inputProps}
                    disabled={disabled}
                    onBlur={this.handleOnBlur}
                    onChange={this.handleOnChange}
                    onFocus={this.handleOnFocus}
                    onKeyDown={this.handleKeyDown}
                    placeholder={placeholder}
                    readOnly={disabled}
                    ref={assignInputRef}
                    type="text"
                    value={value}
                />
                {showDropDownTriangle && showAll()}
                {closeField()}
            </span>
        );
    }
}
