import React, {Component} from 'react';
import R from 'ramda';
import PropTypes from 'prop-types';
import moment from 'moment';
import 'react-datepicker/dist/react-datepicker.css';
import {getModelValue, isComponentDisabled, resetPristinity} from '../../utils/components-utils';
import {localeMessages} from '../../services/locale-service';
import {XlReactWidgetContainerProps} from '../xl-react-widget-props';
import {
    computeInitialState,
    computeNextPropsState,
    computeNextState,
    getValidators
} from '../../services/component-service';
import {genericDateFormat, isDateInRange, toMoment} from '../../utils/date-utils';
import {ERROR} from '../../utils/ci-utils';
import {XlReactWidgetDateComponent} from './xl-react-widget-date-component';

export const getInvalidDateValidators = (dateFormat, msg) => [(model) => {
    const message = {
        show: true,
        text: msg || localeMessages.invalidDateExampleMessage,
        type: ERROR
    };

    const condition = model && !toMoment(model, dateFormat).isValid();
    return {condition, message};
}];

export class XlReactWidgetDate extends Component {
    static propTypes = {
        dateFormat: PropTypes.string,
        invalidDateFormatErrorMessage: PropTypes.string,
        maxDate: PropTypes.object,
        messageHandler: PropTypes.func,
        metadata: PropTypes.object,
        minDate: PropTypes.object,
        model: PropTypes.object,
        showParentMessages: PropTypes.bool,
        ...XlReactWidgetContainerProps
    };

    static defaultProps = {
        dateFormat: genericDateFormat,
        metadata: {},
        validators: []
    };

    constructor(props) {
        super(props);
        this.state = {};
    }

    UNSAFE_componentWillMount() {
        const model = getModelValue(this.props, 'model', new Date());
        computeInitialState(this, this.props, this.getDateValidators(this.props), this.getMomentDate(model));
    }

    UNSAFE_componentWillReceiveProps(nextProps) {
        computeNextPropsState(this, nextProps, this.state, this.getDateValidators(nextProps));
    }

    onChange = (date) => {
        computeNextState(this, this.props, this.getDateValidators(this.props), this.getMomentDate(date));
    };

    onBlur = (event) => {
        const selectedDate = R.pathOr(false, ['target', 'value'], event);
        if (selectedDate) {
            computeNextState(this, this.props, this.getDateValidators(this.props), this.getMomentDate(selectedDate));
        }
    };

    getMomentDate = (date) => toMoment(date, this.props.dateFormat).toDate();

    api = {
        focus: () => this.inputField.focus(),
        resetPristinity: (config) => resetPristinity(this, this.props, config)
    };

    assignRef = (dateWidget) => {
        this.inputField = dateWidget ? dateWidget.input : undefined;
    };

    getDateValidators = (props) => {
        const {dateFormat, invalidDateFormatErrorMessage, maxDate, minDate} = this.props;
        return getValidators(props,
            [...getInvalidDateValidators(dateFormat, invalidDateFormatErrorMessage), (model) => ({
                condition: !isDateInRange({checkedDate: model, maxDate, minDate}),
                message: {
                    show: true,
                    text: localeMessages.dateIsNotAllowed,
                    type: ERROR
                }
            })]);
    };

    render() {
        const {
            className, dateFormat, displayMode, maxDate, minDate, onValidityChange,
            metadata: {description, label, placeholder}
        } = this.props;
        const {message, model, valid} = this.state;
        const disabled = isComponentDisabled(this.props);

        return <XlReactWidgetDateComponent {...({
            assignRef: this.assignRef,
            className,
            dateFormat,
            description,
            disabled,
            displayMode,
            handleModelChange: this.onChange,
            handleOnBlur: this.onBlur,
            label,
            maxDate: maxDate && moment(maxDate).toDate(),
            message,
            minDate: minDate && moment(minDate).toDate(),
            model,
            onValidityChange,
            placeholderText: placeholder,
            valid
        })}
        />;
    }
}
