import R from 'ramda';
import PropTypes from 'prop-types';
import React, {Component} from 'react';
import classNames from 'classnames';
import {Panel} from 'react-bootstrap';
import {localeMessages} from '../../services/locale-service';
import {XlReactWidgetTree} from '../xl-react-widget-tree/xl-react-widget-tree';
import {transformDataForPatchSample} from './xl-react-widget-patch-sample-utils';
import {samplePanelTreeRowRenderer} from './sample-panel-tree-row-renderer';
import './xl-react-widget-patch-sample.less';

export class XlReactWidgetPatchSample extends Component {
    static propTypes = {
        customClassNames: PropTypes.arrayOf(PropTypes.string),
        filePath: PropTypes.string,
        headerLabel: PropTypes.string.isRequired,
        model: PropTypes.string.isRequired,
        onLineSelected: PropTypes.func.isRequired,
        rowRenderer: PropTypes.func,
        selectedLines: PropTypes.arrayOf(PropTypes.string).isRequired
    };

    static defaultProps = {
        customClassNames: [],
        highlightedLineColor: '',
        rowRenderer: samplePanelTreeRowRenderer,
        selectedLines: []
    };

    UNSAFE_componentWillReceiveProps(nextProps) {
        if (!R.equals(nextProps.model, this.props.model)) {
            this.component.tree.loadData(transformDataForPatchSample(nextProps.model));
        }
    }

    render() {
        const {
            customClassNames,
            headerLabel,
            filePath,
            model,
            onLineSelected,
            rowRenderer,
            selectedLines
        } = this.props;

        const assignTree = (component) => {
            if (component) {
                const treeData = transformDataForPatchSample(model);
                component.tree.loadData(treeData);
                this.component = component;
            }
        };

        return <Panel className="xl-react-widget-patch-sample">
            <Panel.Heading className="panel-heading">
                <div className="header-label">{headerLabel}</div>
                {filePath && <div className="file-path">{filePath}</div>}
            </Panel.Heading>
            <Panel.Body className="panel-body">
                <XlReactWidgetTree
                    className={classNames('sample-panel-tree', ...customClassNames)}
                    noDataText={localeMessages.noSample}
                    onNodesSelected={onLineSelected}
                    rowRenderer={rowRenderer}
                    selectedNodes={selectedLines}
                    selectionMode="single"
                    treeRef={assignTree}
                />
            </Panel.Body>
        </Panel>;
    }
}
