import R from 'ramda';
import {actionReducer, composeReducer, stateIdentity} from '../../pages/reducers/reducer-utils';
import xlReactWidgetReduxTableActions from './xl-react-widget-redux-table-actions';

const initializeTable = actionReducer((state, action) => ({
    ...state,
    [action.tableId]: {...R.omit(['api', 'rest'], action), ...action.rest, data: [], total: 0}
}));

const updateSingleFilter = (state, action) =>
    R.reduce(
        (acc, filter) =>
            filter.key === action.filter.change.key ?
                [...acc, {...filter, ...action.filter.change}] :
                [...acc, filter],
        [],
        state[action.tableId].filters
    );

const updateAllFilters = (state, action) =>
    R.reduce(
        (acc, filter) => [...acc, {...filter, model: action.filter.change[filter.key]}],
        [],
        state[action.tableId].filters
    );

const updateFilters = actionReducer((state, action) => {
    const filters = () => action.filter.type === 'single' ?
        updateSingleFilter(state, action) :
        updateAllFilters(state, action);

    return R.cond([
        [R.compose(R.isNil, R.path([action.tableId, 'paging'])), R.evolve({[action.tableId]: {filters}})],
        [R.T, R.evolve({
            [action.tableId]: {
                filters,
                paging: (currPaging) => ({...currPaging, page: 1})
            }
        })]
    ])(state);
});

const updatePagination = actionReducer((state, action) => ({
    ...state,
    [action.tableId]: {
        ...state[action.tableId],
        paging: {
            ...state[action.tableId].paging,
            page: action.page,
            resultsPerPage: action.resultsPerPage
        }
    }
}));

const updateSorting = actionReducer((state, action) => ({
    ...state,
    [action.tableId]: {
        ...state[action.tableId],
        sorting: {
            name: action.name,
            order: action.order
        }
    }
}));

const show = actionReducer((state, action) => ({
    ...state,
    [action.tableId]: {
        ...state[action.tableId],
        ...action
    }
}));

const changeLoadingState = actionReducer((state, action) => ({
    ...state,
    [action.tableId]: {
        ...state[action.tableId],
        loading: action.loading
    }
}));

const mergeContext = actionReducer((state, action) => state[action.tableId] ? {
    ...state,
    [action.tableId]: R.mergeDeepRight(state[action.tableId], action.context)
} : state);

const unmount = actionReducer((state, action) => R.omit([action.tableId], state));

export const xlReactWidgetReduxTableReducer = (initialState = {}) => composeReducer(
    initializeTable(xlReactWidgetReduxTableActions.INITIALIZE),
    changeLoadingState(xlReactWidgetReduxTableActions.CHANGE_LOADING_STATE),
    mergeContext(xlReactWidgetReduxTableActions.MERGE_CONTEXT),
    updateFilters(xlReactWidgetReduxTableActions.SET_FILTERS),
    updatePagination(xlReactWidgetReduxTableActions.SET_PAGING),
    updateSorting(xlReactWidgetReduxTableActions.SET_SORTING),
    show(xlReactWidgetReduxTableActions.SHOW),
    unmount(xlReactWidgetReduxTableActions.UNMOUNT),
    stateIdentity
)(initialState);
