import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import className from 'classnames';

export default class XlReactWidgetTreeSelectItemComponent extends PureComponent {
    static propTypes = {
        indentationPadding: PropTypes.number,
        isFocused: PropTypes.bool,
        node: PropTypes.shape({
            depth: PropTypes.number,
            enabled: PropTypes.bool,
            id: PropTypes.string,
            isExpanded: PropTypes.bool,
            isParent: PropTypes.bool,
            isSelected: PropTypes.bool,
            title: PropTypes.string
        }).isRequired,
        onMouseEnter: PropTypes.func,
        onSelect: PropTypes.func,
        onToggle: PropTypes.func,
        style: PropTypes.object
    };

    static defaultProps = {indentationPadding: 20, isFocused: false};

    handleClick = (event) => {
        const {node: {enabled, id, isSelected}, onSelect} = this.props;
        const shouldSelect = () => onSelect && enabled && !isSelected;
        if (shouldSelect()) {
            onSelect(id, event);
        }
    };

    handleFolderToggle = (event) => {
        const {node: {id}, onToggle} = this.props;
        if (onToggle) {
            onToggle(id, event);
        }
    };

    handleMouseEnter = (event) => {
        const {node: {id}, onMouseEnter} = this.props;
        if (onMouseEnter) {
            onMouseEnter(id, event);
        }
    };

    render() {
        const {
            indentationPadding,
            isFocused,
            node: {
                depth,
                enabled,
                isExpanded,
                isParent,
                isSelected,
                title
            },
            style
        } = this.props;

        const withPadding = {
            ...style,
            paddingLeft: indentationPadding * depth
        };

        const folderIconClassName = className({
            collapsed: !isExpanded,
            expanded: isExpanded
        });

        const treeItemClassName = className('xl-react-widget-tree-item',
            {
                disabled: !enabled,
                focused: isFocused,
                selected: isSelected
            });

        return <div
            className={treeItemClassName}
            onClick={this.handleClick}
            onMouseEnter={this.handleMouseEnter}
            style={withPadding}>
            <span className="xl-react-tree-node">
                {isParent &&
                    <i
                        className={folderIconClassName}
                        onClick={this.handleFolderToggle}
                    />
                }
                <i className="xl-icon folder-icon icon-l"/>
                {title}
            </span>
        </div>;
    }
}
