import R from 'ramda';

export const multiSelectNodes = (currentNodeId, selectedNodes) => R.cond([
    [R.includes(currentNodeId), R.without([currentNodeId])],
    [R.T, (nodes) => R.append(currentNodeId, nodes)]
])(selectedNodes);

export const batchSelectNodes = (batchStartNodeId, toNodeId, selectedNodes, treeNodes) => {
    const isInBoundaries = R.propSatisfies(
        R.complement(R.includes(R.__, [batchStartNodeId, toNodeId])),
        'id'
    );
    return R.pipe(
        R.dropWhile(isInBoundaries),
        R.dropLastWhile(isInBoundaries),
        R.pluck('id'),
        R.concat(selectedNodes),
        R.uniq
    )(treeNodes);
};

const isNodeLoading = (node) => node.state.loading;

export const isNodeOpened = (node) => {
    const hasChildren = node.hasChildren();
    const {state: {open}} = node;
    return hasChildren && open && !isNodeLoading(node);
};

const shouldShowTogglerClass = (node) => {
    const hasChildren = node.hasChildren();
    return !(!hasChildren && !node.loadOnDemand);
};

export const getNodeTogglerClass = (togglerClass, node) => ({
    'infinite-tree-closed': !isNodeOpened(node),
    'infinite-tree-loading': isNodeLoading(node),
    [togglerClass]: shouldShowTogglerClass(node)
});

export const getNodeFromEvent = (event) => {
    const element = $(event.target).closest('.infinite-tree-item');
    return {id: element.data('id'), type: element.data('type')};
};
