import React from 'react';
import classNames from 'classnames';
import PropTypes from 'prop-types';
import {OverlayTrigger, Popover} from 'react-bootstrap';

import './xl-react-widget-with-popup.less';

const ReactWidgetWithPopup = (props) => {
    const popoverWithTitle = (content, title) => <Popover className="xl-react-widget-with-popup" id={props.popoverid}>
        <Popover.Title as="h3">{title}</Popover.Title>
        <Popover.Content>{content}</Popover.Content>
    </Popover>;

    const popoverWithoutTitle =
        (content) => <Popover className="xl-react-widget-with-popup" id={props.popoverid}>
            {content}
        </Popover>;

    const renderPopoverComponent =
        (content, title) => title ? popoverWithTitle(content, title) : popoverWithoutTitle(content);

    const {
        children,
        className,
        delayHide,
        delayShow,
        placement,
        popoverContent,
        popoverTitle,
        trigger,
        ...rest
    } = props;

    const content = <span {...rest} className={classNames('xl-react-widget-with-popup-content', className)}>
        {children}
    </span>;

    return (
        <OverlayTrigger
            delayHide={delayHide}
            delayShow={delayShow}
            overlay={renderPopoverComponent(popoverContent, popoverTitle)}
            placement={placement}
            trigger={trigger}>
            {content}
        </OverlayTrigger>
    );
};

ReactWidgetWithPopup.propTypes = {
    children: PropTypes.node,
    className: PropTypes.string,
    delayHide: PropTypes.number,
    delayShow: PropTypes.number,
    placement: PropTypes.string,
    popoverContent: PropTypes.node.isRequired,
    popoverid: PropTypes.string.isRequired,
    popoverTitle: PropTypes.node,
    trigger: PropTypes.arrayOf(PropTypes.string)
};

ReactWidgetWithPopup.defaultProps = {
    delayHide: 200,
    delayShow: 1200,
    placement: 'top',
    trigger: ['hover', 'focus']
};

export const XlReactWidgetWithPopup = React.memo(ReactWidgetWithPopup);
