import gulp from 'gulp';
import './lint';
import {protractor, webdriver_update} from 'gulp-protractor';
import env from 'gulp-env';
import rimraf from 'rimraf';
import {argv} from 'yargs';
import paths from '../utils/paths';
import port from '../utils/port';
import seleniumVersion from './selenium/selenium-versions.json';

gulp.task('webdriver-cleanup', (cb) => {
    rimraf(`${paths.nodeModulesDir}/webdriver-manager/selenium`, cb);
});

gulp.task('webdriver-update', gulp.series('webdriver-cleanup', (done) => {
    webdriver_update({
        browsers: ['chrome', 'gecko'],
        webdriverManagerArgs: [
            '--versions.standalone',
            seleniumVersion.selenium,
            '--versions.gecko',
            `v${seleniumVersion.geckodriver}`,
            '--versions.chrome',
            seleniumVersion.chromedriver
        ]
    }, done);
}));

gulp.task('webdriver-cleanup', (cb) => {
    rimraf(`${paths.nodeModulesDir}/webdriver-manager/selenium`, cb);
});

export const runProtractor = (cb, envs) => {
    let protractorArgs = ['--params.proxyPort', port.toString()];
    if (argv.suite) {
        protractorArgs = protractorArgs.concat(['--suite', argv.suite]);
    }
    console.log('starting protractor with arguments', protractorArgs);
    return gulp
        .src([`${paths.testDir}/e2e/*.coffee`])
        .pipe(envs)
        .pipe(protractor({
            configFile: `${paths.testsDir}/e2e/config/protractor.conf.js`,
            args: protractorArgs
        }))
        .on('end', () => {
            cb();
            process.exit(0);
        })
        .on('error', (error) => {
            cb();
            throw error;
        });
};

gulp.task('protractor', (cb) => runProtractor(cb, env.set({})));

gulp.task('e2e', gulp.series('lint-tests', 'webdriver-update', 'dev-server', 'protractor'));
