import webpack from 'webpack';
import paths from '../../utils/paths';
import common from './webpack.config.common';

const mode = 'production';

export default {
    entry: {
        colors: ['./app/colors'],
        dip: ['./app/dip-components'],
        index: ['./app/components']
    },
    externals: [
        'biguint-format',
        'classnames',
        'es6-promise',
        'eventsource-polyfill',
        'flake-idgen',
        'jquery',
        'keymirror',
        'moment',
        'monaco-editor',
        'object-assign',
        'postcss-loader',
        'ramda',
        'ramdasauce',
        'react',
        'react-bootstrap',
        'react-bootstrap-table-next',
        'react-datepicker',
        'react-draggable',
        'react-dom',
        'react-hot-loader',
        'react-monaco-editor',
        'react-redux',
        'react-router',
        'react-sortablejs',
        'react-split-pane',
        'react-tagsinput',
        'react-virtualized',
        'react-virtualized-select',
        'redux',
        'redux-devtools',
        'redux-logger',
        'connected-react-router',
        'redux-thunk',
        'xl-infinite-tree',
        'xl-react-fileupload',
        'uuid-v4'
    ],
    mode,
    module: {
        rules: [
            {
                include: /node_modules/,
                loaders: ['strip-sourcemap-loader'],
                test: /\.js$/
            },
            {
                exclude: /(node_modules)/,
                loader: 'babel-loader',
                options: {
                    cacheDirectory: true,
                    plugins: [
                        ['@babel/plugin-proposal-decorators', {legacy: true}],
                        '@babel/plugin-proposal-class-properties',
                        '@babel/plugin-transform-runtime'
                    ]
                },
                test: /\.js?$/
            },
            ...common.rulesConfig
        ]
    },
    optimization: {
        minimize: true,
        splitChunks: {chunks: 'all'}
    },
    output: {
        chunkFilename: '[name].bundle.js',
        filename: '[name].js',
        library: 'xl-react-components',
        libraryTarget: 'commonjs2',
        path: paths.distDir,
        publicPath: '/dist/'
    },
    performance: {
        maxAssetSize: 768000,
        maxEntrypointSize: 768000
    },
    plugins: [
        new webpack.DefinePlugin({'process.env.NODE_ENV': JSON.stringify(mode)}),
        new webpack.optimize.ModuleConcatenationPlugin()
    ]
};
