import json from 'format-json';
import {Action, ActionUtil, ElementUtil, Expectation, WaitCondition} from 'protractor-base-dsl';

const sleepTime = 300;

export const changeDisplayMode = (modeName) => {
    const selector = `.testBindings .display-mode input[type="radio"].${modeName}`;
    const action = () => Action.jsClick(selector);
    const condition = () => WaitCondition.displayed(`${selector}:checked`);
    ActionUtil.repeatAction(action, condition);
};

export const toggleCheckbox = (selector) =>
    ElementUtil.elementFinder(selector).getAttribute('checked').then((checked) => {
        const action = () => Action.jsClick(selector);
        const condition = () => WaitCondition.checkboxChecked(selector, !checked);
        ActionUtil.repeatAction(action, condition);
    });

export const displayModeCreation = () => changeDisplayMode('creation');

export const displayModeEdit = () => changeDisplayMode('edit');

export const displayModeDisplay = () => changeDisplayMode('display');

export const expectDefaultValueToEqual = (value) =>
    Expectation.textEquals('.testBindings .default-value input', value);

export const expectDefaultValueButtonToBeVisible = () =>
    Expectation.displayed('.testBindings .default-value .apply-value-btn');

export const expectPristineToEqual = (value) =>
    Expectation.textEquals('.testBindings .value.pristine', value);

export const expectValidToEqual = (value) =>
    Expectation.textEquals('.testBindings .value.valid', value);

export const expectModelToContain = (value) =>
    Expectation.textContains('.testBindings .value.model', value.toString());

export const expectObjectModelToEqual = (model) =>
    Expectation.textEquals('.testBindings .model.value', json.plain(model));

export const expectModelToEqual = (value) =>
    Expectation.textEquals('.testBindings .value.model', value.toString());

export const focus = () =>
    Action.jsClick('.testBindings .focus-btn');

const openEnumSelection = (fieldName) => {
    const containerSelector = `.testBindings .change-${fieldName}`;
    const action = () => Action.click(`${containerSelector} .show-all`);
    const condition = () => WaitCondition.present(`${containerSelector} .resize-triggers`);
    return ActionUtil.repeatAction(action, condition);
};

const openEnumAndSelect = (optionNum, fieldName) => {
    openEnumSelection(fieldName);
    const suggestionOption = `.xl-react-autocomplete-suggestion:nth-child(${optionNum}) div`;
    Action.click(`.testBindings .change-${fieldName} .xl-react-autocomplete-suggestion-list ${suggestionOption}`);
    return browser.driver.sleep(sleepTime);
};

export const resetPristinity = () =>
    Action.jsClick('.testBindings .reset-pristinity-btn');

export const setDefaultValue = (defaultValue) => {
    const inputSelector = '.testBindings .default-value input';
    Action.clearText(inputSelector);
    Action.typeText(inputSelector, defaultValue);
    Expectation.textEquals(inputSelector, defaultValue);
    Action.jsClick('.testBindings .default-value .apply-value-btn');
};

export const show = () =>
    toggleCheckbox('.testBindings input.show-modal');

export const toggleRequired = () =>
    toggleCheckbox('.testBindings input.required');

export const toggleOnlyUniqueSuggestion = () =>
    toggleCheckbox('.testBindings .only-unique-suggestion');

export const toggleDisabled = () =>
    toggleCheckbox('.testBindings input.xl-disabled');

export const toggleShowMessages = () =>
    toggleCheckbox('.testBindings input.show-messages');

export const setInputType = (inputType) =>
    Action.click(`.testBindings .input-type input[value="${inputType}"`);

export const togglePassword = () => toggleCheckbox('.testBindings input.password');

const TestBindings = {
    changeDisplayMode,
    displayModeCreation,
    displayModeDisplay,
    displayModeEdit,
    expectDefaultValueToEqual,
    expectDefaultValueButtonToBeVisible,
    expectValidToEqual,
    expectModelToContain,
    expectObjectModelToEqual,
    expectPristineToEqual,
    expectModelToEqual,
    focus,
    setInputType,
    openEnumAndSelect,
    openEnumSelection,
    resetPristinity,
    setDefaultValue,
    show,
    toggleCheckbox,
    toggleOnlyUniqueSuggestion,
    toggleDisabled,
    togglePassword,
    toggleRequired,
    toggleShowMessages
};

export default TestBindings;
global.TestBindings = TestBindings;
