import {Action, Expectation, ReactAction} from 'protractor-base-dsl';
import * as KeyCodes from 'keycode-js';

export const clickEnterOnDialog = () =>
    ReactAction.keyDown('.xl-react-widget-confirmation-dialog', KeyCodes.KEY_RETURN);
export const clickEscapeOnDialog = () =>
    ReactAction.clickEscape('.xl-react-widget-confirmation-dialog');
export const clickOnKey = (key) =>
    ReactAction.keyDown('.xl-react-widget-confirmation-dialog', KeyCodes[key]);
export const clickOnPrimaryButton = () =>
    Action.click('.xl-react-modal-dialog .dialog-buttons .xl-primary');
export const clickOnSecondaryButton = () =>
    Action.click('.xl-react-modal-dialog .dialog-buttons .xl-secondary');
export const clickOnDisabledButton = () =>
    Action.clickIfClickable('.xl-react-modal-dialog .dialog-buttons button[disabled]');
export const clickOnCheckbox = () =>
    Action.click(".xl-react-modal-dialog .dialog-buttons input[type='checkbox']");
export const selectSkin = (value) => {
    const radioGroupSelector = '.xl-react-widget-confirmation-dialog-default .testBindings .radioButtonGroup';
    const skinSelector = `${radioGroupSelector} input[name="skin"][value="${value}"]`;
    return Action.click(skinSelector);
};
export const selectCheckboxSkin = () => selectSkin('checkbox');
export const expectHeader = (headerText) =>
    Expectation.textEquals('.xl-react-modal-dialog .xl-react-modal-content .modal-header h4', headerText);
export const expectContent = (contentText) =>
    Expectation.textEquals('.xl-react-modal-dialog .confirmation-dialog-text', contentText);
export const expectNotPresent = () =>
    Expectation.notPresent('.xl-react-modal-dialog');

const WidgetConfirmationDialog = {
    clickEnterOnDialog,
    clickEscapeOnDialog,
    clickOnKey,
    clickOnPrimaryButton,
    clickOnSecondaryButton,
    clickOnDisabledButton,
    clickOnCheckbox,
    expectContent,
    expectHeader,
    expectNotPresent,
    selectCheckboxSkin,
    selectSkin
};

export default WidgetConfirmationDialog;
global.WidgetConfirmationDialog = WidgetConfirmationDialog;
