import {Action, Expectation} from 'protractor-base-dsl';
import MonacoEditor from '../../common/monaco-editor';
import WidgetPatchSample from './widget-patch-sample';

const controlIconSelector = '.xl-react-widget-patch-editor .xl-icon';
const editorLineSelector = '.editor-panel .view-lines .view-line span.mtk1';
const modalSelector = '.xl-react-widget-patch-editor-confirmation-dialog';
const sampleContainerSelector = '.xl-react-widget-patch-editor .sample-panel-tree';

const changePatchedValue = (text) => MonacoEditor.changeText(text);

const clickOnDeleteOperation = () => Action.click(`${controlIconSelector}.delete-icon`);
const clickOnEditOperation = () => Action.click(`${controlIconSelector}.edit-icon`);

const applyPatchEntry = () => {
    Action.click(`${modalSelector} .xl-react-button.xl-primary`);
};

const expectConfirmationDialog = () => {
    Expectation.displayed(modalSelector);
    Expectation.textEquals(`${modalSelector} .title`, 'Discard changes');
};

const expectLastedPushedPatch = (expectedText) => {
    Expectation.textEquals('.xl-react-widget-patch-editor .testBindings .last-pushed-patch.value', expectedText);
};

const isDeleteOperationActive = () => Expectation.displayed(`${controlIconSelector}.add.active`);
const isEditOperationActive = () => Expectation.displayed(`${controlIconSelector}.add.active`);

const isPristine = () =>
    Expectation.textEquals('.xl-react-widget-patch-editor .testBindings .pristine.value', 'true');

const isNotPristine = () =>
    Expectation.textEquals('.xl-react-widget-patch-editor .testBindings .pristine.value', 'false');

const isSampleInDeletingMode = () => Expectation.displayed(`${sampleContainerSelector}.remove`);
const isSampleInEditingMode = () => Expectation.displayed(`${sampleContainerSelector}.replace`);

const selectedPathIs = (expectedPath) =>
    Expectation.textEquals('.xl-react-widget-patch-editor .selected-path', expectedPath);

const patchedFirstLineContains = (expectedText) =>
    Expectation.textContains(editorLineSelector, expectedText);

const selectLineInEditor = (lineNum, expectedPath) => {
    WidgetPatchSample.selectLine(lineNum);
    WidgetPatchSample.expectedSelectedLine(lineNum);
    selectedPathIs(expectedPath);
};

const WidgetPatchEditor = {
    applyPatchEntry,
    changePatchedValue,
    clickOnDeleteOperation,
    clickOnEditOperation,
    expectConfirmationDialog,
    expectLastedPushedPatch,
    isDeleteOperationActive,
    isEditOperationActive,
    isNotPristine,
    isPristine,
    isSampleInDeletingMode,
    isSampleInEditingMode,
    patchedFirstLineContains,
    selectLineInEditor,
    selectedPathIs
};

export default WidgetPatchEditor;
global.WidgetPatchEditor = WidgetPatchEditor;
