import {Action, ActionUtil, Expectation, WaitCondition} from 'protractor-base-dsl';
import XlComponent from '../base/xl-component';
import TestBindings from '../../common/test-bindings';

const sleepTime = 300;
const componentSelector = '.testComponent .xl-react-components';
const inputSelector = `${componentSelector}.xl-react-widget-string input`;
const textareaSelector = `${componentSelector}.xl-react-widget-string textarea`;

export const clearSelectedSize = () =>
    Action.click('.testBindings .change-size .close-icon');

export const clearValue = () =>
    Action.click('.testComponent .close-icon');

export const clickOnTextArea = () =>
    Action.click(textareaSelector);

export const clickShowPasswordButton = () =>
    Action.click('.testComponent .visibility-on-icon');

export const clickUnshowPasswordButton = () =>
    Action.click('.testComponent .visibility-off-icon');

export const expectClearIconPresent = () =>
    Expectation.displayed('.testComponent .close-icon');

export const expectShowPasswordIconPresent = () =>
    Expectation.displayed('.testComponent .visibility-on-icon');

export const expectClearIconNotPresent = () =>
    Expectation.notDisplayed('.testComponent .close-icon');

export const expectShowPasswordIconNotPresent = () =>
    Expectation.notDisplayed('.testComponent .visibility-on-icon');

export const expectTextAreaPlaceholderToEqual = (placeholderValue) =>
    Expectation.attributeContainsValue(textareaSelector, 'placeholder', placeholderValue);

export const expectFirstLineMonacoFieldHasText = (text) => {
    const firstLineSelector = `${componentSelector} .view-lines .view-line:nth-child(1) span span`;
    Expectation.textEquals(firstLineSelector, text);
};

export const expectMonacoFieldReadonly = () =>
    Expectation.attributeContainsValue(
        `${componentSelector} .xl-react-component-input-wrapper`, 'class', 'readonly');

export const expectMonacoFieldNotReadonly = () =>
    Expectation.attributeDoesntContainValue(
        `${componentSelector} .xl-react-component-input-wrapper`, 'class', 'readonly');

export const expectTextAreaContentToEqual = (content) =>
    Expectation.textEquals(textareaSelector, content);

export const expectInputContentToEqual = (content) =>
    Expectation.textEquals(inputSelector, content);

export const hideChangedValue = () => {
    const action = () => XlComponent.clickOnLabel();
    const condition = () => WaitCondition.textEquals(textareaSelector, '*(changed) encrypted value*');
    ActionUtil.repeatAction(action, condition);
};

export const expectPlaceholderToEqual = (placeholderValue) =>
    Expectation.attributeContainsValue(inputSelector, 'placeholder', placeholderValue);

export const expectReadonlyInput = () =>
    Expectation.hasAttribute(inputSelector, 'readonly');

export const expectNotReadonlyInput = () =>
    Expectation.hasNoAttribute(inputSelector, 'readonly');

export const expectReadonlyTextArea = () =>
    Expectation.hasAttribute(textareaSelector, 'readonly');

export const expectNotReadonlyTextArea = () =>
    Expectation.hasNoAttribute(textareaSelector, 'readonly');

export const expectInputTypeToEqual = (type) =>
    Expectation.attributeContainsValue(inputSelector, 'type', type);

export const openTypeSelect = () => TestBindings.openEnumSelection('type');

export const toggleSmallSize = () => TestBindings.openEnumAndSelect(3, 'size');

export const toggleMediumSize = () => TestBindings.openEnumAndSelect(2, 'size');

export const toggleLargeSize = () => TestBindings.openEnumAndSelect(1, 'size');

export const toggleDefaultView = () => TestBindings.openEnumAndSelect(1, 'view');

export const toggleMonacoView = () => TestBindings.openEnumAndSelect(2, 'view');

export const toggleSearch = () => {
    openTypeSelect();
    const suggestionOption = '.xl-react-autocomplete-suggestion:nth-child(2) div';
    Action.click(`.testBindings .change-type .xl-react-autocomplete-suggestion-list ${suggestionOption}`);
    browser.driver.sleep(sleepTime);
};

export const setValue = (value) =>
    Action.typeText(inputSelector, value);

export const setTextAreaValue = (value) =>
    Action.typeText(textareaSelector, value);

const WidgetString = {
    ...XlComponent,
    clearSelectedSize,
    clearValue,
    clickOnTextArea,
    clickShowPasswordButton,
    clickUnshowPasswordButton,
    expectClearIconNotPresent,
    expectClearIconPresent,
    expectFirstLineMonacoFieldHasText,
    expectMonacoFieldReadonly,
    expectMonacoFieldNotReadonly,
    expectShowPasswordIconPresent,
    expectInputContentToEqual,
    expectShowPasswordIconNotPresent,
    expectInputTypeToEqual,
    expectNotReadonlyInput,
    expectNotReadonlyTextArea,
    expectPlaceholderToEqual,
    expectReadonlyInput,
    expectReadonlyTextArea,
    expectTextAreaContentToEqual,
    expectTextAreaPlaceholderToEqual,
    hideChangedValue,
    setValue,
    setTextAreaValue,
    toggleDefaultView,
    toggleLargeSize,
    toggleMediumSize,
    toggleMonacoView,
    toggleSearch,
    toggleSmallSize
};

export default WidgetString;
global.WidgetString = WidgetString;
