import Navigation from '../../dsl/common/navigation';
import TestBindings from '../../dsl/common/test-bindings';
import DipBoolean from '../../dsl/component/dip/dip-boolean';

describe('XL DIP Boolean Scenario', () => {
    afterAll(() => TestBindings.resetPristinity());
    beforeAll(() => Navigation.openDipElement('dipBoolean1'));

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual('false');
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        DipBoolean.expectLabel("I'm a boolean");
        DipBoolean.expectDescription('This is my check field.');
    });

    it('should modify internal values when checkbox is clicked', () => {
        DipBoolean.clickCheckbox();
        TestBindings.expectModelToEqual('true');
        TestBindings.expectPristineToEqual('false');
        TestBindings.expectValidToEqual('true');
    });

    it('should be disabled when "disabled" option is chosen', () => {
        DipBoolean.expectEnabled();
        TestBindings.toggleDisabled();
        DipBoolean.expectDisabled();
        TestBindings.toggleDisabled();
        DipBoolean.expectEnabled();
    });

    it('should behave properly in different display modes', () => {
        TestBindings.displayModeDisplay();
        DipBoolean.expectDisabled();
        TestBindings.displayModeCreation();
        DipBoolean.expectEnabled();
        TestBindings.displayModeEdit();
        DipBoolean.expectEnabled();
    });
});
