import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import DipInteger from '../../../dsl/component/dip/dip-integer';

describe('XL DIP Integer Scenario', () => {
    afterAll(() => TestBindings.resetPristinity());
    beforeAll(() => Navigation.openDipElement('dipInteger1'));

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual('');
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        DipInteger.expectLabel('I\'m an integer');
    });

    it('should be able to modify internal values', () => {
        DipInteger.setValue(10);
        TestBindings.expectModelToEqual(10);
        TestBindings.expectPristineToEqual('false');
        TestBindings.expectValidToEqual('true');
    });

    it('should show warnings', () => {
        TestBindings.toggleShowMessages();
        DipInteger.expectDescription('Sample Warning');
        TestBindings.toggleShowMessages();
    });

    it('should be disabled when "disabled" option is chosen', () => {
        DipInteger.expectEnabled();
        TestBindings.toggleDisabled();
        DipInteger.expectDisabled();
        TestBindings.toggleDisabled();
        DipInteger.expectEnabled();
    });

    it('should behave properly in different display modes', () => {
        TestBindings.displayModeDisplay();
        DipInteger.expectDisabled();
        TestBindings.displayModeCreation();
        DipInteger.expectEnabled();
        TestBindings.displayModeEdit();
        DipInteger.expectEnabled();
    });

    it('should display cross if any text is typed in input field', () => {
        DipInteger.setValue(10);
        DipInteger.expectClearIconPresent();
        TestBindings.expectPristineToEqual('false');
        DipInteger.clearValue();
        TestBindings.expectPristineToEqual('false');
        DipInteger.expectClearIconNotPresent();
        TestBindings.expectModelToEqual('');
    });
});
