import R from 'ramda';
import json from 'format-json';
import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import DipMapStringString from '../../../dsl/component/dip/dip-map-string-string';

describe('XL Dip Map String String Scenario', () => {
    beforeAll(() => {
        Navigation.openDipElement('dipMapStringString1');
    });

    afterAll(() => {
        TestBindings.resetPristinity();
    });

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual(json.plain({
            Name: 'Marc',
            Surname: 'Dupont',
            Age: 39,
            Street: '433 Boulevard des templiers',
            Zipcode: 74003,
            City: 'Paris',
            Phone: '0123456789',
            'Mobile Phone': '+31123456789'
        }));

        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        DipMapStringString.expectDescription('Find the description here');
        DipMapStringString.expectLabel('Dictionary');
        DipMapStringString.expectFilterPresent();
        DipMapStringString.expectAddButtonEnabled();
        DipMapStringString.expectNoErrorMessage();

        DipMapStringString.expectKeyCellValue(1, 'Age');
        DipMapStringString.expectValueCellValue(1, '39');

        DipMapStringString.expectKeyCellValue(2, 'City');
        DipMapStringString.expectValueCellValue(2, 'Paris');

        DipMapStringString.expectKeyCellValue(3, 'Mobile Phone');
        DipMapStringString.expectValueCellValue(3, '+31123456789');

        DipMapStringString.expectKeyCellValue(4, 'Name');
        DipMapStringString.expectValueCellValue(4, 'Marc');

        DipMapStringString.expectKeyCellValue(5, 'Phone');
        DipMapStringString.expectValueCellValue(5, '0123456789');
    });

    it('should auto focus key filed while adding a new row', () => {
        DipMapStringString.expectAddButtonEnabled();
        DipMapStringString.clickAddButton();
        DipMapStringString.expectKeyInputFocused();
    });

    it('should be able to sort asc/desc by click column header', () => {
        DipMapStringString.clickOnKeyHeader();

        DipMapStringString.expectKeyCellValue(1, 'Age');
        DipMapStringString.expectKeyCellValue(2, 'City');
        DipMapStringString.expectKeyCellValue(3, 'Mobile Phone');

        DipMapStringString.clickOnKeyHeader();

        DipMapStringString.expectKeyCellValue(1, 'Zipcode');
        DipMapStringString.expectKeyCellValue(2, 'Surname');
        DipMapStringString.expectKeyCellValue(3, 'Street');

        DipMapStringString.clickOnKeyHeader();
    });

    it('should show error if key is duplicated', () => {
        DipMapStringString.clickAddButton();
        DipMapStringString.setKeyValue(1, 'Age');
        DipMapStringString.expectDuplicatedKey('Age');
        DipMapStringString.removeEntry(1);
        DipMapStringString.expectNoErrorMessage();
    });

    it('should filter items', () => {
        DipMapStringString.expectFilterPresent();
        DipMapStringString.setFilterValue('st');

        DipMapStringString.expectKeyCellValue(1, 'Street');
        DipMapStringString.expectValueCellValue(1, '433 Boulevard des templiers');
        DipMapStringString.clearFilter();

        DipMapStringString.expectKeyCellValue(1, 'Age');
        DipMapStringString.expectValueCellValue(1, '39');
    });

    it('should filter when disabled', () => {
        TestBindings.toggleDisabled();
        DipMapStringString.setFilterValue('st');
        DipMapStringString.expectKeyCellValue(1, 'Street');
        DipMapStringString.expectValueCellValue(1, '433 Boulevard des templiers');
        DipMapStringString.clearFilter();
        TestBindings.toggleDisabled();
    });

    it('should filter when in visual display mode', () => {
        TestBindings.displayModeDisplay();
        DipMapStringString.setFilterValue('st');
        DipMapStringString.expectKeyCellValue(1, 'Street');
        DipMapStringString.expectValueCellValue(1, '433 Boulevard des templiers');
        DipMapStringString.clearFilter();
        TestBindings.displayModeEdit();
    });

    it('should be readonly when disabled or in visual display mode', () => {
        TestBindings.toggleDisabled();
        DipMapStringString.expectAddButtonPresent();
        DipMapStringString.expectRemoveEntryNotPresent(1);

        DipMapStringString.expectReadonlyKeyCell(1);
        DipMapStringString.expectReadonlyValueCell(1);

        TestBindings.displayModeDisplay();
        DipMapStringString.expectAddButtonPresent();
        DipMapStringString.expectRemoveEntryNotPresent(1);
        DipMapStringString.expectReadonlyKeyCell(1);
        DipMapStringString.expectReadonlyValueCell(1);

        TestBindings.toggleDisabled();
        DipMapStringString.expectAddButtonPresent();
        DipMapStringString.expectRemoveEntryNotPresent(1);
        DipMapStringString.expectReadonlyKeyCell(1);
        DipMapStringString.expectReadonlyValueCell(1);

        TestBindings.displayModeCreation();
    });

    it('should be able to remove items', () => {
        DipMapStringString.removeEntry(1);
        DipMapStringString.removeEntry(1);
        DipMapStringString.removeEntry(1);
        DipMapStringString.removeEntry(1);

        TestBindings.expectPristineToEqual('false');

        DipMapStringString.expectKeyCellValue(1, 'Phone');
        DipMapStringString.expectValueCellValue(1, '0123456789');

        DipMapStringString.expectKeyCellValue(2, 'Street');
        DipMapStringString.expectValueCellValue(2, '433 Boulevard des templiers');

        DipMapStringString.expectKeyCellValue(3, 'Surname');
        DipMapStringString.expectValueCellValue(3, 'Dupont');
    });

    it('should be able to add items', () => {
        DipMapStringString.expectAddButtonEnabled();
        DipMapStringString.clickAddButton();
        DipMapStringString.setEntry(1, 'Akey', 'Avalue');
        DipMapStringString.expectAddButtonEnabled();
        DipMapStringString.clickOnFilter();

        DipMapStringString.expectKeyCellValue(1, 'Akey');
        DipMapStringString.expectValueCellValue(1, 'Avalue');
        DipMapStringString.expectAddButtonEnabled();
    });

    it('should show error message when no values and component is required to have a value', () => {
        TestBindings.toggleRequired();
        DipMapStringString.expectDescription('Find the description here');
        R.forEach(() => DipMapStringString.removeEntry(1), R.range(1, 6));
        DipMapStringString.expectError('This field is required');
        TestBindings.toggleRequired();
    });
});
