import R from 'ramda';
import Navigation from '../../dsl/common/navigation';
import TestBindings from '../../dsl/common/test-bindings';
import DipSetOfCi from '../../dsl/component/dip/dip-set-of-ci';

describe('XL DIP Set Of Ci Scenario', () => {
    afterAll(() => TestBindings.resetPristinity());
    beforeAll(() => Navigation.openDipElement('dipSetOfCi1'));

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual('["Item 1","Item 2","Item 3","Item 4","Item 5"]');
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        DipSetOfCi.expectDescription('Find the description here');
        DipSetOfCi.expectLabel('Set of CIs');
        DipSetOfCi.expectEmptyInputValue();

        for (const index of R.range(1, 6)) {
            DipSetOfCi.expectSelectedItemToEqual(index, `Item ${index}`);
        }

        DipSetOfCi.expectSelectedItemNotPresent(6);
        DipSetOfCi.expectClearIconOfItemToBePresent(1);
        DipSetOfCi.expectClearIconOfItemToBePresent(3);
    });

    it('should allow to add and remove elements (No duplicates)', () => {
        DipSetOfCi.openSelection();
        DipSetOfCi.expectSuggestionListIsPresent();

        for (const index of R.range(1, 6)) {
            DipSetOfCi.expectSuggestionValue(index, `Application${index}`);
        }

        DipSetOfCi.selectOption(1);
        DipSetOfCi.expectInvalidItem(1);
        DipSetOfCi.expectValidItem(6);
        DipSetOfCi.openSelection();
        DipSetOfCi.selectOption(2);
        DipSetOfCi.openSelection();
        DipSetOfCi.selectOption(2);
        DipSetOfCi.expectSuggestionListNotPresent();
        DipSetOfCi.expectEmptyInputValue();
        TestBindings.expectModelToEqual('["Application1","Application2","Item 1","Item 2","Item 3","Item 4","Item 5"]');

        DipSetOfCi.expectSelectedItemToEqual(1, 'Application1');
        DipSetOfCi.expectSelectedItemToEqual(2, 'Application2');
        DipSetOfCi.expectSelectedItemToEqual(3, 'Item 1');
        DipSetOfCi.expectSelectedItemToEqual(4, 'Item 2');
        DipSetOfCi.expectSelectedItemToEqual(5, 'Item 3');
        DipSetOfCi.expectSelectedItemToEqual(6, 'Item 4');
        DipSetOfCi.expectSelectedItemToEqual(7, 'Item 5');
        DipSetOfCi.expectSelectedItemNotPresent(8);

        DipSetOfCi.removeSelectedItem(1);
        DipSetOfCi.expectSelectedItemToEqual(1, 'Application2');

        DipSetOfCi.removeSelectedItem(4);
        DipSetOfCi.expectSelectedItemToEqual(2, 'Item 1');
        DipSetOfCi.expectSelectedItemToEqual(3, 'Item 2');
        DipSetOfCi.expectSelectedItemToEqual(4, 'Item 4');
        DipSetOfCi.expectSelectedItemToEqual(5, 'Item 5');
        DipSetOfCi.expectSelectedItemNotPresent(6);
        TestBindings.expectModelToEqual('["Application2","Item 1","Item 2","Item 4","Item 5"]');
    });

    it('should allow to filter elements and remove them while filtered', () => {
        DipSetOfCi.setFilter('2');
        DipSetOfCi.expectSelectedItemToEqual(1, 'Application2');
        DipSetOfCi.expectSelectedItemToEqual(2, 'Item 2');
        DipSetOfCi.expectSelectedItemNotPresent(3);
        TestBindings.expectModelToEqual('["Application2","Item 1","Item 2","Item 4","Item 5"]');
        TestBindings.expectPristineToEqual('false');
        TestBindings.expectValidToEqual('true');
        DipSetOfCi.removeSelectedItem(2);
        DipSetOfCi.expectSelectedItemToEqual(1, 'Application2');
        DipSetOfCi.expectSelectedItemNotPresent(2);
        TestBindings.expectModelToEqual('["Application2","Item 1","Item 4","Item 5"]');
        DipSetOfCi.clearFilter();
        DipSetOfCi.expectSelectedItemToEqual(1, 'Application2');
        DipSetOfCi.expectSelectedItemToEqual(2, 'Item 1');
        DipSetOfCi.expectSelectedItemToEqual(3, 'Item 4');
        DipSetOfCi.expectSelectedItemToEqual(4, 'Item 5');
        DipSetOfCi.expectSelectedItemNotPresent(5);
    });

    it('should disable elements in disabled mode but still allow to search', () => {
        DipSetOfCi.expectComponentEnabled();
        TestBindings.toggleDisabled();
        DipSetOfCi.expectComponentDisabled();
        TestBindings.toggleDisabled();
        DipSetOfCi.expectComponentEnabled();
    });

    it('should properly render different view modes', () => {
        TestBindings.displayModeDisplay();
        DipSetOfCi.expectComponentDisabled();
        TestBindings.displayModeCreation();
        DipSetOfCi.expectComponentEnabled();
        TestBindings.displayModeEdit();
        DipSetOfCi.expectComponentEnabled();
    });
});
