import Form from '../../dsl/component/form/form';
import Navigation from '../../dsl/common/navigation';
import TestBindings from '../../dsl/common/test-bindings';
import MonacoEditor from '../../dsl/common/monaco-editor';

describe('XL Dip Dynamic Form Empty Required With Warnings Scenario', () => {
    beforeAll(() => {
        Navigation.openFormElement('dynamicFormEmptyWithWarnings');
    });

    afterAll(() => TestBindings.resetPristinity());

    it('should have all fields valid', () => {
        Form.expectAllFieldsAreValid();
        Form.expectAllFieldsHaveDescription();
    });

    it('should give priority to errors over warnings', () => {
        TestBindings.toggleShowMessages();
        TestBindings.toggleRequired();
        Form.expectInvalidCategory(1);

        Form.expectErrorMessage('Label component String', 'This field is required');
        Form.expectErrorMessage('Label component StringWithTypeHint', 'This field is required');
        Form.expectErrorMessage('Label component StringLarge', 'This field is required');
        Form.expectErrorMessage('Label component StringMonaco', 'This field is required');
        Form.expectErrorMessage('Label component Integer', 'This field is required');
        Form.expectErrorMessage('Label component Set', 'This field is required');
        Form.expectErrorMessage('Label component List', 'This field is required');
        Form.expectErrorMessage('Label component CI', 'This field is required');
        Form.expectErrorMessage('Label String enum', 'This field is required');
        Form.expectErrorMessage('Label Radio enum', 'This field is required');
        Form.expectErrorMessage('Label component Map String String', 'This field is required');
        Form.expectErrorMessage('Label component Set Of CI', 'This field is required');
        Form.expectErrorMessage('Label component List Of CI', 'This field is required');
        TestBindings.toggleRequired();
        TestBindings.toggleShowMessages();
    });

    it('should show warnings during typing', () => {
        Form.setInputValue('Label component String', 'com');
        Form.expectWarningBorder('Label component String');
        Form.expectWarningMessage('Label component String', 'Value should start with comp');

        Form.setInputValue('Label component StringWithTypeHint', 'test');
        Form.cleanValue('Label component StringWithTypeHint');
        Form.expectWarningBorder('Label component StringWithTypeHint');
        Form.expectWarningMessage('Label component StringWithTypeHint', 'Value should contain `str` in the value');

        Form.setInputValue('Label component String', 'puter');
        Form.expectWarningBorder('Label component String');
        Form.expectWarningMessage('Label component String', 'Value should end on 0');
        Form.setInputValue('Label component String', '0');
        const description = 'Description component String (Property: nameComponentString)';
        Form.expectDescription('Label component String', description);

        Form.setTextAreaValue('Label component StringLarge', 'st');
        Form.expectWarningBorder('Label component StringLarge');
        Form.expectWarningMessage('Label component StringLarge', 'Value should contain `str` in the value');
        Form.setTextAreaValue('Label component StringLarge', 'str');
        Form.expectDescription('Label component StringLarge',
            'Description component StringLarge (Property: nameComponentStringLarge)');

        MonacoEditor.changeText('mon');
        Form.expectWarningMessage('Label component StringMonaco', 'Value should contain `monaco editor` in the value');

        Form.setInputValue('Label component Integer', '6');
        const warningMessage = 'Value should contain digits between 0 and 5 inclusively';
        Form.expectWarningBorder('Label component Integer');
        Form.expectWarningMessage('Label component Integer', warningMessage);
        Form.cleanValue('Label component Integer');
        Form.setInputValue('Label component Integer', '35');
        Form.expectDescription('Label component Integer',
            'Description component Integer (Property: nameComponentInteger)');

        Form.setInputValue('Label component CI', 'A');
        Form.expectWarningBorder('Label component CI');
        Form.expectWarningMessage('Label component CI', 'Value should start with App');
        Form.setInputValue('Label component CI', 'pp');
        Form.expectDescription('Label component CI', 'Description component CI (Property: nameComponentCi)');
        Form.cleanValue('Label component CI');

        Form.setInputValue('Label String enum', 'aaaaaa');
        Form.expectWarningBorder('Label String enum');
        Form.expectWarningMessage('Label String enum', 'Value should be less than 5 symbols');
        Form.cleanValue('Label String enum');
        Form.setInputValue('Label String enum', 'luigi');
        Form.expectDescription('Label String enum', 'Description enum of string (Property: nameStringEnum)');

        Form.setRadioValue('Label Radio enum', 3);
        Form.expectWarningMessage('Label Radio enum', 'Value should be less than 5 symbols');
        Form.setRadioValue('Label Radio enum', 5);
        Form.expectDescription('Label Radio enum', 'Description enum of strings as radio (Property: nameRadioEnum)');
    });
});
