import Form from '../../dsl/component/form/form';
import Navigation from '../../dsl/common/navigation';
import TestBindings from '../../dsl/common/test-bindings';

describe('XL Dip Dynamic Form Empty Scenario', () => {
    beforeAll(() => {
        Navigation.openFormElement('dynamicFormEmpty');
    });

    afterAll(() => TestBindings.resetPristinity());

    it('should properly display default values', () => {
        Form.expectFieldHasValue('Label component String', 'input', '');
        Form.expectFieldHasValue('Label component StringWithTypeHint', 'input', '');
        Form.expectFieldHasText('Label component StringLarge', 'textarea', '');
        Form.expectFirstLineMonacoFieldHasText('Label component StringMonaco', '');
        Form.expectFieldHasValue('Label component Integer', 'input', '');
        Form.expectDipSetOfStringNoTags('Label component Set');
        Form.expectDipCollectionHasNoValues('Label component List');
        Form.expectFieldHasValue('Label component CI', 'input', '');
        Form.expectDipMapIsEmpty('Label component Map String String');
        Form.expectDipCollectionHasNoValues('Label component Set Of CI');
        Form.expectDipCollectionHasNoValues('Label component List Of CI');
        Form.expectDateTimeToEqual('Label component Date Time', 'Nov 9, 2016', '3:10 PM');
    });

    it('should have correct descriptions', () => {
        Form.expectAllFieldsHaveDescription();
    });

    it('should have all fields pristine and valid', () => {
        Form.expectAllFieldsArePristine();
        Form.expectAllFieldsAreValid();
    });

    it('should have proper model', () => {
        Form.expectModelToEqual({
            nameComponentString: '',
            nameComponentStringWithTypeHint: '',
            nameComponentStringLarge: '',
            nameComponentStringMonaco: '',
            nameComponentInteger: '',
            nameComponentBoolean: false,
            nameComponentSet: [],
            nameComponentList: [],
            nameComponentCi: '',
            nameStringEnum: '',
            nameRadioEnum: '',
            mapStringStringName: {},
            nameSetOfCi: [],
            nameListOfCi: [],
            datetimeName: 'November 9th 2016, 3:10:00 pm',
            datetimeStringName: 'November 9th 2017, 3:10:00 pm'
        });
    });

    it('should properly behave in display mode', () => {
        TestBindings.displayModeDisplay();
        Form.expectFieldReadonly('input', 'Label component String');
        Form.expectFieldDisabled('Label component StringWithTypeHint');
        Form.expectFieldReadonly('textarea', 'Label component StringLarge');
        Form.expectMonacoFieldReadonly('Label component StringMonaco');
        Form.expectFieldDisabled('Label component Integer');
        Form.expectFieldDisabled('Label component Boolean');
        Form.expectDipStringsDisabled('Label component Set');
        Form.expectFieldDisabled('Label component List', 'input');
        Form.expectFieldDisabled('Label component CI');
        Form.expectFieldDisabled('Label String enum');
        Form.expectFieldDisabled('Label Radio enum');
        Form.expectDipMapDisabled('Label component Map String String');
        TestBindings.displayModeCreation();
    });

    it('should properly behave when form disabled', () => {
        TestBindings.toggleDisabled();
        Form.expectFieldReadonly('input', 'Label component String');
        Form.expectFieldDisabled('Label component StringWithTypeHint');
        Form.expectFieldReadonly('textarea', 'Label component StringLarge');
        Form.expectMonacoFieldReadonly('Label component StringMonaco');
        Form.expectFieldDisabled('Label component Integer');
        Form.expectFieldDisabled('Label component Boolean');
        Form.expectDipStringsDisabled('Label component Set');
        Form.expectFieldDisabled('Label component List', 'input');
        Form.expectFieldDisabled('Label component CI');
        Form.expectFieldDisabled('Label String enum');
        Form.expectFieldDisabled('Label Radio enum');
        Form.expectDipMapDisabled('Label component Map String String');
        TestBindings.toggleDisabled();
    });

    it('should highlight incorrect date field', () => {
        Form.setInputValue('Label component Date Time', 'a');
        Form.clickOnFieldLabel('Label component Date Time');
        Form.expectErrorMessage('Label component Date Time', 'Example: Dec 1, 2018');
        Form.expectFieldInvalid('Label component Date Time');
    });
});

