import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import WidgetConfirmationDialog from '../../../dsl/component/widget/widget-confirmation-dialog';

const dialogTitle = 'Confirmation dialog with a long title name which doesn\'t fit the default size of modal window';
const dialogContent = 'Are you sure you want to leave? Your changes will be lost.';

describe('XL Widget Confirmation Dialog Scenario', () => {
    beforeAll(() => Navigation.openWidgetElement('widgetConfirmationDialog1'));

    it('should be able to open and close modal dialog', () => {
        TestBindings.show();
        WidgetConfirmationDialog.expectHeader(dialogTitle);
        WidgetConfirmationDialog.expectContent(dialogContent);
        WidgetConfirmationDialog.clickOnPrimaryButton();
        WidgetConfirmationDialog.expectNotPresent();

        TestBindings.show();
        WidgetConfirmationDialog.expectHeader(dialogTitle);
        WidgetConfirmationDialog.clickOnSecondaryButton();
        WidgetConfirmationDialog.expectNotPresent();
    });

    it('should be able perform Primary/Secondary actions with ENTER/ESC keys', () => {
        TestBindings.show();
        WidgetConfirmationDialog.expectHeader(dialogTitle);
        WidgetConfirmationDialog.expectContent(dialogContent);
        WidgetConfirmationDialog.clickOnKey('KEY_RETURN');
        WidgetConfirmationDialog.expectNotPresent();

        TestBindings.show();
        WidgetConfirmationDialog.expectHeader(dialogTitle);
        WidgetConfirmationDialog.clickOnKey('KEY_ESCAPE');
        WidgetConfirmationDialog.expectNotPresent();
    });

    it('should not be able perform actions on a disabled button', () => {
        TestBindings.show();
        WidgetConfirmationDialog.expectHeader(dialogTitle);
        WidgetConfirmationDialog.expectContent(dialogContent);
        WidgetConfirmationDialog.clickOnDisabledButton();
        WidgetConfirmationDialog.expectContent(dialogContent);
        WidgetConfirmationDialog.clickOnKey('KEY_0');
        WidgetConfirmationDialog.expectContent(dialogContent);

        WidgetConfirmationDialog.clickOnKey('KEY_ESCAPE');
    });

    it('should be able to perform actions on checkbox skin', () => {
        WidgetConfirmationDialog.selectCheckboxSkin();
        TestBindings.show();
        WidgetConfirmationDialog.expectHeader(dialogTitle);
        WidgetConfirmationDialog.expectContent(dialogContent);
        WidgetConfirmationDialog.clickOnCheckbox();
        WidgetConfirmationDialog.expectNotPresent();
    });
});
