import Navigation from '../../../dsl/common/navigation';
import WidgetPatchSample from '../../../dsl/component/widget/widget-patch-sample';
import WidgetPatchEditor from '../../../dsl/component/widget/widget-patch-editor';

describe('XL React Widget Patch Editor Scenario', () => {
    beforeAll(() => Navigation.openWidgetElement('widgetPatchEditor'));

    it('should have right initial state', () => {
        WidgetPatchSample.expectedNoSelectedLine();
        WidgetPatchSample.expectLineContent(1, 'apiVersion: v1');
        WidgetPatchSample.expectLineContent(2, 'kind: Pod');
        WidgetPatchSample.expectHeaderLabel('Sample');
        WidgetPatchSample.expectFilePath('Environments/Patch-1');
    });

    it('should be able to select the line and change operation', () => {
        WidgetPatchEditor.selectLineInEditor(1, '/');
        WidgetPatchEditor.patchedFirstLineContains('apiVersion: v1');
        WidgetPatchEditor.selectLineInEditor(3, '/');

        WidgetPatchEditor.isSampleInEditingMode();

        WidgetPatchEditor.clickOnDeleteOperation();
        WidgetPatchEditor.isSampleInDeletingMode();

        WidgetPatchEditor.clickOnEditOperation();
        WidgetPatchEditor.isSampleInEditingMode();

        WidgetPatchEditor.selectLineInEditor(12, '/spec/containers[0]');
        WidgetPatchEditor.patchedFirstLineContains('command: ["/bin/sh"]');
    });

    it('should ask about applying changes', () => {
        WidgetPatchEditor.selectLineInEditor(4, '/metadata');
        WidgetPatchEditor.isPristine();
        WidgetPatchEditor.changePatchedValue('a');
        WidgetPatchEditor.isNotPristine();
        WidgetPatchEditor.clickOnEditOperation();

        WidgetPatchSample.selectLine(2);
        WidgetPatchEditor.expectConfirmationDialog();
        WidgetPatchEditor.applyPatchEntry();
        WidgetPatchEditor.isPristine();
        WidgetPatchEditor.expectLastedPushedPatch('a');
    });
});
