import variables from '../../../../../app/styles/variables';
import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import WidgetTagsAutocomplete from '../../../dsl/component/widget/widget-tags-autocomplete';
import WidgetTags from '../../../dsl/component/widget/widget-tags';
import Form from '../../../dsl/component/form/form';

const longId = 'ZApp very very very very very very very very very very very very very very very very very ' +
    'very very very very very very very very very very very very long name';

describe('XL Widget Tags With Autocomplete Scenario', () => {
    beforeAll(() => Navigation.openWidgetElement('widgetTags3'));

    afterAll(() => TestBindings.resetPristinity());

    it('should have right initial state', () => {
        Form.expectModelToEqual([
            {
                color: variables['xl-green'],
                label: 'Application 0',
                value: 'Applications/TestApp/Application 0'
            }
        ]);
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        WidgetTagsAutocomplete.expectEmptyInputValue();

        WidgetTagsAutocomplete.expectDescription('These are autocomplete tags.');
        WidgetTagsAutocomplete.expectLabel('Tags with autocomplete');
        WidgetTagsAutocomplete.expectAddButtonNotPresent();
        WidgetTagsAutocomplete.clickOnContainer();
        WidgetTagsAutocomplete.expectSuggestionListIsPresent();
        WidgetTagsAutocomplete.expectTagPresent(1, 'Applications/TestApp/Application 0');
        WidgetTagsAutocomplete.expectClearForTagPresent(1);
        WidgetTagsAutocomplete.expectCheckboxChecked(1);
        WidgetTagsAutocomplete.expectTagNotPresent(2);
        WidgetTagsAutocomplete.expectCheckboxUnChecked(2);
    });

    it('it should allow to add new tags', () => {
        WidgetTagsAutocomplete.clickOnContainer();
        WidgetTagsAutocomplete.setValue('1');
        WidgetTagsAutocomplete.expectSuggestionListIsPresent();
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        WidgetTagsAutocomplete.expectSuggestionValue(1, 'Applications/TestApp/Application 1');
        WidgetTagsAutocomplete.selectOption(1);
        Form.expectModelToEqual([
            {
                color: variables['xl-green'],
                label: 'Application 0',
                value: 'Applications/TestApp/Application 0'
            },
            {
                color: variables['xl-green'],
                label: 'Application 1',
                value: 'Applications/TestApp/Application 1'
            }
        ]);
        WidgetTagsAutocomplete.clickOnTitle();
        WidgetTagsAutocomplete.expectSuggestionListNotPresent();
        TestBindings.expectPristineToEqual('false');
        TestBindings.expectValidToEqual('true');
        WidgetTagsAutocomplete.expectEmptyInputValue();
        WidgetTagsAutocomplete.expectTagPresent(1, 'Applications/TestApp/Application 0');
        WidgetTagsAutocomplete.expectClearForTagPresent(1);
        WidgetTagsAutocomplete.expectTagPresent(2, 'Applications/TestApp/Application 1');
        WidgetTagsAutocomplete.expectClearForTagPresent(2);
        WidgetTagsAutocomplete.expectTagNotPresent(3);
    });

    it('should not allow to do anything when component is disabled', () => {
        TestBindings.toggleDisabled();
        WidgetTagsAutocomplete.expectTagPresent(1, 'Applications/TestApp/Application 0');
        WidgetTagsAutocomplete.expectClearForTagNotPresent(1, 'Applications/TestApp/Application 0');
        WidgetTagsAutocomplete.expectTagPresent(2, 'Applications/TestApp/Application 1');
        WidgetTagsAutocomplete.expectClearForTagNotPresent(2, 'Applications/TestApp/Application 1');
        WidgetTagsAutocomplete.expectInputDisabled();
        TestBindings.toggleDisabled();
        WidgetTagsAutocomplete.expectTagPresent(1, 'Applications/TestApp/Application 0');
        WidgetTagsAutocomplete.expectClearForTagPresent(1);
        WidgetTagsAutocomplete.expectTagPresent(2, 'Applications/TestApp/Application 1');
        WidgetTagsAutocomplete.expectClearForTagPresent(2);
        WidgetTagsAutocomplete.expectInputEnabled();
    });

    it('should allow to remove tags', () => {
        WidgetTags.removeTag(2);
        WidgetTagsAutocomplete.expectTagNotPresent(2);
        WidgetTagsAutocomplete.expectTagPresent(1, 'Applications/TestApp/Application 0');
        WidgetTagsAutocomplete.expectClearForTagPresent(1);
    });

    it('should behave different in different view modes', () => {
        TestBindings.displayModeCreation();
        WidgetTagsAutocomplete.expectInputEnabled();
        WidgetTagsAutocomplete.expectTagPresent(1, 'Applications/TestApp/Application 0');
        WidgetTagsAutocomplete.expectClearForTagPresent(1);
        WidgetTagsAutocomplete.expectDescription('These are autocomplete tags.');

        TestBindings.displayModeDisplay();
        WidgetTagsAutocomplete.expectInputDisabled();
        WidgetTagsAutocomplete.expectTagPresent(1, 'Applications/TestApp/Application 0');
        WidgetTagsAutocomplete.expectClearForTagNotPresent(1, 'Applications/TestApp/Application 0');
        WidgetTagsAutocomplete.expectDescription('These are autocomplete tags.');

        TestBindings.displayModeEdit();
        WidgetTagsAutocomplete.expectInputEnabled();
        WidgetTagsAutocomplete.expectTagPresent(1, 'Applications/TestApp/Application 0');
        WidgetTagsAutocomplete.expectClearForTagPresent(1);
        WidgetTagsAutocomplete.expectDescription('These are autocomplete tags.');
    });

    it('should be able to select all items by Select all button', () => {
        WidgetTagsAutocomplete.openSuggestions();
        WidgetTagsAutocomplete.clickSelectAll();
        TestBindings.expectModelToContain('Applications/TestApp/Application 15');
        TestBindings.expectModelToContain(longId);
        TestBindings.expectPristineToEqual('false');
        TestBindings.expectValidToEqual('true');
        WidgetTagsAutocomplete.expectEmptyInputValue();

        WidgetTagsAutocomplete.expectTagPresent(1, 'Applications/TestApp/Application 0');
        WidgetTagsAutocomplete.expectClearForTagPresent(1);

        WidgetTagsAutocomplete.expectTagPresent(2, 'Applications/TestApp/Application 1');
        WidgetTagsAutocomplete.expectClearForTagPresent(2);

        WidgetTagsAutocomplete.expectTagPresent(3, 'Applications/TestApp/Application 10');
        WidgetTagsAutocomplete.expectClearForTagPresent(3);
        WidgetTagsAutocomplete.expectTagNotPresent(43);
    });

    it('should be close the suggestions only when clicking outside', () => {
        WidgetTagsAutocomplete.openSuggestions();
        WidgetTagsAutocomplete.clickClearAll();
        WidgetTagsAutocomplete.clickOnContainer();
        WidgetTagsAutocomplete.setValue('9');

        WidgetTagsAutocomplete.selectOptionAndExpectTag(1, 'Applications/TestApp/Application 19');
        WidgetTagsAutocomplete.expectClearForTagPresent(1);
        WidgetTagsAutocomplete.expectCheckboxChecked(1);
        WidgetTagsAutocomplete.expectSuggestionListIsPresent();

        TestBindings.toggleOnlyUniqueSuggestion();
        WidgetTagsAutocomplete.clickOnLabel();
        WidgetTagsAutocomplete.openSuggestions();

        WidgetTagsAutocomplete.setValue('9');
        WidgetTagsAutocomplete.selectOptionAndExpectTag(2, 'Applications/TestApp/Application 29');
        WidgetTagsAutocomplete.expectClearForTagPresent(2);
        WidgetTagsAutocomplete.expectCheckboxChecked(2);
        WidgetTagsAutocomplete.expectSuggestionListIsPresent();

        WidgetTagsAutocomplete.selectOptionAndExpectTag(3, 'Applications/TestApp/Application 39');
        WidgetTagsAutocomplete.expectClearForTagPresent(3);
        WidgetTagsAutocomplete.expectCheckboxChecked(3);
        WidgetTagsAutocomplete.expectSuggestionListIsPresent();

        Form.expectModelToEqual([
            {
                color: variables['xl-green'],
                label: 'Application 19',
                value: 'Applications/TestApp/Application 19'
            },
            {
                color: variables['xl-green'],
                label: 'Application 29',
                value: 'Applications/TestApp/Application 29'
            },
            {
                color: variables['xl-green'],
                label: 'Application 39',
                value: 'Applications/TestApp/Application 39'
            }
        ]);

        WidgetTagsAutocomplete.selectOption(2);
        WidgetTagsAutocomplete.selectOption(3);
        WidgetTagsAutocomplete.clickOnTitle();
    });

    it('should be able to add free typed value', () => {
        WidgetTagsAutocomplete.clickOnContainer();
        WidgetTagsAutocomplete.addFreeTypedValue('aaa');
        WidgetTagsAutocomplete.expectTagPresent(2, 'aaa');
        WidgetTagsAutocomplete.openSuggestions();
        WidgetTagsAutocomplete.expectSuggestionValue(1, 'aaa');
        WidgetTagsAutocomplete.clickOnTitle();
        WidgetTags.removeTag(2);
        WidgetTagsAutocomplete.openSuggestions();
        WidgetTagsAutocomplete.expectSuggestionValue(1, 'Applications/TestApp/Application 0');
    });

    it('should be able to display different skins', () => {
        WidgetTagsAutocomplete.expectCheckboxSkin();
        WidgetTagsAutocomplete.expectNoColorAndCheckboxSkin();
        WidgetTagsAutocomplete.clickOnTitle();

        WidgetTagsAutocomplete.selectColorAndCheckboxSkin();
        WidgetTagsAutocomplete.expectColorAndCheckboxSkin();
        WidgetTagsAutocomplete.clickOnTitle();

        WidgetTagsAutocomplete.selectCheckboxSkin();
        WidgetTagsAutocomplete.expectCheckboxSkin();
        WidgetTagsAutocomplete.expectNoColorAndCheckboxSkin();
    });

    it('should be able to expand and collapse when there are more tags', () => {
        WidgetTagsAutocomplete.openSuggestions();
        WidgetTags.expectRemainingTagsLabelNotPresent();
        WidgetTagsAutocomplete.clickSelectAll();
        WidgetTags.expectRemainingTagsLabelPresent();
        WidgetTags.removeTag(1);
        WidgetTagsAutocomplete.clickOnTitle();
        WidgetTags.expectRemainingTagsLabelPresent(34);
        WidgetTagsAutocomplete.openSuggestions();
        WidgetTags.expectRemainingTagsLabelNotPresent();
    });

    it('should be able to show tags when clicking on the placeholder', () => {
        WidgetTagsAutocomplete.openSuggestions();
        WidgetTagsAutocomplete.clickClearAll();
        WidgetTagsAutocomplete.clickOnPlaceholder();
        WidgetTagsAutocomplete.clickSelectAll();
        TestBindings.expectModelToContain('Applications/TestApp/Application 15');
        TestBindings.expectModelToContain(longId);
    });
});
