import Navigation from '../../../dsl/common/navigation';
import WidgetTree from '../../../dsl/component/widget/widget-tree';

describe('XL Widget Tree Scenario', () => {
    beforeAll(() => Navigation.openWidgetElement('widgetTree1'));

    it('should be able to open/close single nodes', () => {
        WidgetTree.expectNode(1, 'node--1');
        WidgetTree.expectNodeCollapsed(1);
        WidgetTree.toggleNode(1);
        WidgetTree.expectNodeExpanded(1);
        WidgetTree.toggleNode(1);
        WidgetTree.expectNodeCollapsed(1);
    });

    it('should be able to select single nodes', () => {
        WidgetTree.expectNodeNotSelected(1);
        WidgetTree.selectNode(1);
        WidgetTree.expectNodeSelected(1);
    });

    it('should be able to multi-select/deselect using the ctrl key', () => {
        WidgetTree.toggleNode(1);
        WidgetTree.expectNodeExpanded(1);
        WidgetTree.selectNode(1);
        WidgetTree.expectNodeSelected(1);
        WidgetTree.bulkCtrlClickNodes([1, 2, 3, 4]);
        WidgetTree.expectNodeNotSelected(1);
        WidgetTree.expectNodeSelected(2);
        WidgetTree.expectNodeSelected(3);
        WidgetTree.expectNodeSelected(4);

        WidgetTree.bulkCtrlClickNodes([4]);
        WidgetTree.expectNodeNotSelected(1);
        WidgetTree.expectNodeSelected(2);
        WidgetTree.expectNodeSelected(3);
        WidgetTree.expectNodeNotSelected(4);

        WidgetTree.selectNode(2);
        WidgetTree.expectNodeNotSelected(1);
        WidgetTree.expectNodeSelected(2);
        WidgetTree.expectNodeNotSelected(3);
        WidgetTree.expectNodeNotSelected(4);

        WidgetTree.bulkCtrlClickNodes([2]);
        WidgetTree.toggleNode(1);
        WidgetTree.expectNodeCollapsed(1);
    });

    it('should be able to batch-select using the shift key', () => {
        WidgetTree.expectNodeCollapsed(1);
        WidgetTree.toggleNode(1);
        WidgetTree.expectNodeExpanded(1);

        WidgetTree.selectNode(1);
        WidgetTree.expectNodeSelected(1);

        WidgetTree.bulkShiftClickNodes([4]);
        WidgetTree.expectNodeSelected(1);
        WidgetTree.expectNodeSelected(2);
        WidgetTree.expectNodeSelected(3);
        WidgetTree.expectNodeSelected(4);

        WidgetTree.bulkCtrlClickNodes([2]);
        WidgetTree.expectNodeSelected(1);
        WidgetTree.expectNodeNotSelected(2);
        WidgetTree.expectNodeSelected(3);
        WidgetTree.expectNodeSelected(4);

        WidgetTree.bulkCtrlClickNodes([2]);
        WidgetTree.expectNodeSelected(1);
        WidgetTree.expectNodeSelected(2);
        WidgetTree.expectNodeSelected(3);
        WidgetTree.expectNodeSelected(4);

        WidgetTree.selectNode(1);
        WidgetTree.expectNodeSelected(1);
        WidgetTree.expectNodeNotSelected(2);
        WidgetTree.expectNodeNotSelected(3);
        WidgetTree.expectNodeNotSelected(4);

        WidgetTree.bulkShiftClickNodes([3]);
        WidgetTree.expectNodeSelected(1);
        WidgetTree.expectNodeSelected(2);
        WidgetTree.expectNodeSelected(3);
        WidgetTree.expectNodeNotSelected(4);
    });
});
