import * as service from '~/app/xl-react-dip/xl-react-dip-set-of-strings/xl-react-dip-set-of-strings-service';

describe('xl-dip-set-of-strings', () => {
    const model = ['cat', 'pony', 'donkey'];

    describe('getModelKeys', () => {
        it('should return keys without sorting', () => {
            expect(service.getModelKeys(model)).toEqual(['cat', 'pony', 'donkey']);
        });
        it('should return sorted keys', () => {
            expect(service.getModelKeys(model, true)).toEqual(['cat', 'donkey', 'pony']);
        });
    });
    describe('filterModelKeys', () => {
        it('should filter keys by search input', () => {
            expect(service.filterModel(model, 'o', false)).toEqual(
                ['pony', 'donkey']
            );
        });
    });
    describe('getModelWithUpdatedKey', () => {
        it('should update a value in the model and return a new model', () => {
            const expectedResult = ['zebra', 'pony', 'donkey']; // eslint-disable-line sort-keys
            expect(service.getModelWithUpdatedKey(model, 'cat', 'zebra')).toEqual(expectedResult);
        });
        it('should not do any changes if value is not found', () => {
            expect(service.getModelWithUpdatedKey(model, 'dog', 'zebra')).toEqual(model);
        });
    });
    describe('tableSortMethod', () => {
        it('should sort values based on table sort', () => {
            expect(service.tableSortComparator('Aaa', 'Zzz')).toEqual(-1);
            expect(service.tableSortComparator('Zzz', 'Aaa')).toEqual(1);
            expect(service.tableSortComparator(undefined, 'Aaa', true)).toEqual(1);
            expect(service.tableSortComparator(undefined, 'Aaa', false)).toEqual(0);
            expect(service.tableSortComparator('Azz', undefined, true)).toEqual(0);
            expect(service.tableSortComparator('Azz', undefined, false)).toEqual(0);
        });
    });

    describe('sortTableData', () => {
        it('should sort data for table', () => {
            expect(service.sortData(['Item 1', 'Item 2', 'Item 3', 'Item 4'], 'ASC'))
                .toEqual(['Item 1', 'Item 2', 'Item 3', 'Item 4']);

            expect(service.sortData(['Item 1', 'Item 2', 'Item 3', 'Item 4'], 'DESC'))
                .toEqual(['Item 4', 'Item 3', 'Item 2', 'Item 1']);
        });
    });
});
