import R from 'ramda';
import React from 'react';
import {mount} from 'enzyme';

import Helper from '../validations-spec-helper';
import {XlReactDipDynamicForm} from '~/app/xl-react-form/xl-react-dip-dynamic-form';
import {ERROR} from '~/app/utils/ci-utils';

describe('DIP form', () => {
    const mergeContext = R.F;
    const toggleCategoryGroup = R.F;
    const updateExternalProperties = R.F;

    describe('External validation', () => {
        const mkField = (description, kind, name) => ({description, kind, name});

        it('should show message on correct field', () => {
            const errors = {field1: 'this is wrong'};
            const validator = (fieldName) => ({
                condition: R.has(fieldName, errors),
                message: {show: false, text: R.prop(fieldName, errors), type: ERROR}
            });

            const properties = [
                mkField('This is field1', 'STRING', 'field1'),
                mkField('This is field2', 'INTEGER', 'field2')
            ];

            const props = {
                groupedProperties: [{category: 'Default', isOpen: true, properties}],
                mergeContext,
                metadata: {properties},
                model: {
                    field1: 'value1',
                    field2: 20
                },
                pristinity: {
                    field1: true,
                    field2: true
                },
                toggleCategoryGroup,
                updateExternalProperties,
                validators: [validator]
            };

            const wrapper = mount(<XlReactDipDynamicForm {...props}/>);
            const {component: component1, description: description1, makeUnpristine: makeUnpristine1} =
                new Helper('.xl-react-widget-string').buildFromWrapper(wrapper);
            const {component: component2, description: description2, makeUnpristine: makeUnpristine2} =
                new Helper('.xl-react-dip-integer').buildFromWrapper(wrapper);
            expect(component1()).toBeDefined();
            expect(component2()).toBeDefined();
            expect(description1()).toHaveText('This is field1 (Property: field1)');
            expect(description2()).toHaveText('This is field2 (Property: field2)');

            makeUnpristine1();
            makeUnpristine2();
            setTimeout(() => {
                expect(description1()).toHaveText('this is wrong');
                expect(description2()).toHaveText('This is field2 (Property: field2)');
            }, 500);
        });

        it('should invoke validator for all fields', () => {
            const validator = jest.fn();
            validator.mockImplementation(() => R.T);

            const properties = [
                mkField('This is field1', 'STRING', 'field1'),
                mkField('This is field2', 'INTEGER', 'field2'),
                mkField('This is field3', 'STRING', 'field3')
            ];

            const props = {
                groupedProperties: [{category: 'Default', isOpen: true, properties}],
                mergeContext,
                metadata: {properties},
                model: {
                    field1: 'value1',
                    field2: 20,
                    field3: 'value3'
                },
                pristinity: {
                    field1: true,
                    field2: true,
                    field3: true
                },
                toggleCategoryGroup,
                updateExternalProperties,
                validators: [validator]
            };

            mount(<XlReactDipDynamicForm {...props}/>);
            setTimeout(() => {
                expect(validator).toBeCalledWith('value1');
                expect(validator).toBeCalledWith(20);
                expect(validator).toBeCalledWith('value3');
            }, 500);
        });
    });
});
